@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -42,16 +42,16 @@
 #include <locale.h>
 
 #include <uim/uim.h>
-#include <uim/uim-helper.h>
 
 #include "debug.h"
 #include "output.h"
+#include "uim-el-types.h"
+#include "context.h"
 #include "candidate.h"
 #include "preedit.h"
+#include "helper.h"
 #include "im.h"
-#include "context.h"
 #include "commit.h"
-#include "helper-server.h"
 #include "prop.h"
 
 void commit_cb(void *ptr, const char *str);
@@ -66,8 +66,10 @@ void candidate_shift_page_cb(void *ptr, int direction);
 void candidate_deactivate_cb(void *ptr);
 
 void prop_list_update_cb(void *ptr, const char *str);
-void prop_label_update_cb(void *ptr, const char *str);
 
 void configuration_changed_cb(void *ptr);
 
+void switch_app_global_im_cb(void *ptr, const char *name);
+void switch_system_global_im_cb(void *ptr, const char *name);
+
 #endif
