@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -36,6 +36,17 @@
 
 #include "candidate.h"
 
+candidate_info *
+create_candidate()
+{
+  candidate_info *cand;
+
+  cand = (candidate_info *)malloc(sizeof(candidate_info));
+  cand->valid = 0;
+
+  return cand;
+}
+
 int
 new_candidate(uim_context context, candidate_info *cand, int num, int limit)
 {
@@ -44,10 +55,9 @@ new_candidate(uim_context context, candidate_info *cand, int num, int limit)
 
   if (cand->valid) clear_candidate(cand);
 
-
   cand->valid = 1;
 
-  cand->index = 0; 
+  cand->index = -1;
   cand->disp_limit = limit;
   cand->num = num;
 
@@ -65,14 +75,17 @@ new_candidate(uim_context context, candidate_info *cand, int num, int limit)
 }
 
 
-void
+int
 show_candidate(candidate_info *cand)
 {
   int i;
   int page;
   int index;
 
-  if (!cand->valid || cand->num == 0) return;
+  if (cand->num == 0) {
+	a_printf("( e )");
+	return 0;
+  }
 
   index = cand->index;
   page = index / cand->disp_limit;
@@ -102,6 +115,8 @@ show_candidate(candidate_info *cand)
   }
 
   a_printf(") ");
+
+  return 1;
 }
 
 
