@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -41,10 +41,7 @@
 
 #include <uim/uim.h>
 
-typedef struct uim_key{
-  int mod;
-  int key;
-} uim_key;
+#include "uim-el-types.h"
 
 void convert_keyname_a2e(char *keyname, const char *name);
 enum UKey convert_keyname_e2u( const char *keyname );
