@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -43,10 +43,8 @@
 
 #include "debug.h"
 
-extern int output_enable;
-
-int a_putchar(int c);
-int a_printf(const char *fmt, ...);
+#define a_printf printf
+#define a_putchar putchar
 
 void output_with_escape(const char *str);
 
