@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -43,78 +43,86 @@
 #include <unistd.h>
 #include <locale.h>
 
-#include <sys/select.h>
-
 #include <uim/uim.h>
-#include <uim/uim-helper.h>
 #include <uim/uim-util.h>
 #include <uim/uim-im-switcher.h>
 
 #include "uim-el-agent.h"
 
 
+
+/* called when owner buffer is killed  */
 static int
 cmd_release(int context_id)
 {
-  if (release_uim_agent_context(context_id) < 0)
-	return -1;
-  else
-	return 1;
+  return release_uim_agent_context(context_id);
 }
 
 
+/* process helper message received by uim-el-helper-agent */
+static int
+cmd_helper(int context_id, char *helper_message)
+{
+  uim_agent_context *ua;
+  int ret;
+
+  ua = get_uim_agent_context(context_id);
+  ret = helper_handler(ua, helper_message);
+
+  if (ua && ret) {
+    show_commit_string_uim_agent_context(current);
+    show_preedit_uim_agent_context(ua);
+    show_candidate_uim_agent_context(ua);
+
+    check_prop_list_update(ua);
+    check_default_engine();
+
+    return ret;
+  }
+
+  return 0;
+}
+
+
+/*
+ * remove Emacs local focus from current context
+ */
 static int
 cmd_unfocused(int context_id)
 {
-  int ret;
-  uim_agent_context *ua;
+  uim_agent_context *ua = get_uim_agent_context(context_id);
 
-  /* if context_id is 0, 
-	 this function unfocuses from current focused context 
-	 otherwise,
-	 compare current focused context and unfocuses if they are matching.
-  */
-
-  if (current != NULL) {
-	ua = current;
-	if (context_id != 0 && get_uim_agent_context(context_id) != current)
-	  ret = -1;
-	else
-	  ret = unfocused();
+  if (ua == current) {
+	/* keep preedit if exists */
+	show_preedit_uim_agent_context(ua);
+	show_candidate_uim_agent_context(ua);
+	check_default_engine();
+	return clear_current_uim_agent_context();
   } else {
-	/* already unfocused */
-	if ((ua = get_uim_agent_context(context_id)))
-	  ret = context_id;
-	else
-	  ret = -1;
-  }
-	
-  if (ret > 0)
-	if (context_id != 0) {
-	  show_preedit(ua->pe);
-	  return 1;
-	} else {
-	  return 1;
-	}
-
-  else
+	/* error */
 	return -1;
+  }
 }
 
 
+/*
+ * set Emacs local focus to the context
+ */
 static int
 cmd_focused(int context_id)
 {
   uim_agent_context *ua = get_uim_agent_context(context_id);
 
-  if (focused(ua) > 0) {
-	if (show_preedit(ua->pe))
-	  show_candidate(ua->pe->cand);
+  if (set_current_uim_agent_context(ua) > 0) {
+	show_preedit_uim_agent_context(ua);
+	show_candidate_uim_agent_context(ua);
+
+	check_prop_list_update(ua);
+	check_default_engine();
 	return 1;
   } else {
 	return -1;
   }
-
 }
 
 static int
@@ -128,14 +136,15 @@ cmd_hide(int context_id)
 	return -1;
 }
 
+
 static int
 cmd_show(int context_id)
 {
   uim_agent_context *ua = get_uim_agent_context(context_id);
-  
+
   if (ua != NULL) {
-	if (show_preedit(ua->pe))
-	  show_candidate(ua->pe->cand);
+	show_preedit_uim_agent_context(ua);
+	show_candidate_uim_agent_context(ua);
 	return 1;
   } else {
 	return -1;
@@ -146,15 +155,24 @@ cmd_show(int context_id)
 static int
 cmd_new(int context_id, const char *encoding)
 {
+
+  uim_agent_context *ua;
+
   if (get_uim_agent_context(context_id)) {
 	debug_printf(DEBUG_WARNING, "context %d already exists\n", context_id);
 	return -1;
-  } else {
-	if (new_uim_agent_context(context_id, encoding) != NULL)
-	  return 1;
-	else
-	  return -1;
   }
+
+  ua = new_uim_agent_context(context_id, encoding);
+
+  if (ua) {
+	check_prop_list_update(ua);
+	check_default_engine();
+	return 1;
+  } else {
+	return -1;
+  }
+
 }
 
 
@@ -167,8 +185,12 @@ cmd_reset(int context_id)
   if (ua != NULL) {
 	/* before reset, clear preedit and candidate */
 	clear_preedit(ua->pe);
-	clear_candidate(ua->pe->cand);
+	clear_candidate(ua->cand);
+
 	uim_reset_context(ua->context);
+
+	check_prop_list_update(ua);
+	check_default_engine();
 	return 1;
   } else {
 	return -1;
@@ -177,17 +199,19 @@ cmd_reset(int context_id)
 
 
 static int
-cmd_change(int context_id, const char *im, const char *encoding)
+cmd_change(int context_id, const char *im)
 {
   uim_agent_context *ua;
 
-  if (im && encoding && strlen(im) > 0 
+  if (im && strlen(im) > 0 
 	  && (ua = get_uim_agent_context(context_id))) {
 
-	focused(ua);
-
 	if (check_im_name(im)) {
-	  switch_context_im(ua, im, encoding);
+	  switch_context_im(ua, im);
+
+	  check_prop_list_update(ua);
+	  check_default_engine();
+
 	  return 1;
 	} else {
 	  return -1;	 
@@ -201,6 +225,8 @@ cmd_change(int context_id, const char *im, const char *encoding)
 static int
 cmd_list(void)
 {
+  output_default_im_engine();
+
   if (list_im_engine() > 0)
 	return 1;
   else
@@ -209,8 +235,22 @@ cmd_list(void)
 
 
 static int
-cmd_setenc(const char *im, const char *encoding)
+cmd_setenc(char *opt)
 {
+  char *im, *encoding;
+
+  im = opt;
+  if (im == NULL)
+	return -1;
+
+  if ((encoding = strchr(opt, ' ')) == NULL || *(encoding + 1) == '\0') {
+	/* 2nd arg not found */
+	return -1;
+  } else {
+	*encoding = '\0';
+	encoding ++;
+  }
+
   if (set_im_encoding(im, encoding) > 0)
 	return 1;
   else
@@ -239,6 +279,12 @@ cmd_prop(int context_id, const char *prop)
   if (prop && strlen(prop) > 0 &&
 	  (ua = get_uim_agent_context(context_id))) {
 	uim_prop_activate(ua->context, prop);
+
+	show_im_uim_agent_context(ua);
+	show_prop_uim_agent_context(ua);
+	show_preedit_uim_agent_context(ua);
+	show_candidate_uim_agent_context(ua);
+
 	return 1;
   } else {
 	return -1;
@@ -249,16 +295,37 @@ cmd_prop(int context_id, const char *prop)
 static int
 cmd_label(int context_id)
 {
+  uim_agent_context *ua;
 
-  if (current == NULL)
+  if ((ua = get_uim_agent_context(context_id))) {
+	show_im_uim_agent_context(ua);
+	show_prop_uim_agent_context(ua);
+	show_preedit_uim_agent_context(ua);
+	show_candidate_uim_agent_context(ua);
+	return 1;
+  } else {
 	return -1;
-  else 
-	output_prop_list(current->prop, current->im);
-
-  return 1;
+  }
 }
 
 
+static int
+cmd_nop(int context_id)
+{
+  uim_agent_context *ua = get_uim_agent_context(context_id);
+
+  if (ua) {
+	show_im_uim_agent_context(ua);
+	show_prop_uim_agent_context(ua);
+	show_preedit_uim_agent_context(ua);
+	show_candidate_uim_agent_context(ua);
+	return 1;
+  } else {
+	return -1;
+  }  
+
+  return 1;
+}
 
 static int
 cmd_error(void)
@@ -278,19 +345,17 @@ check_default_engine(void)
 
 
 static void
-check_prop_list_label(void)
+check_prop_list_update(uim_agent_context *ua)
 {
-  if (current == NULL) return;
+  debug_printf(DEBUG_NOTE, "check_prop_list_update\n");
 
-  if (current->prop->list_update) {
-	announce_prop_list_update(current->prop, current->encoding);
-	output_prop_list(current->prop, current->im);
-	current->prop->list_update = 0;
-  }
+  if (ua && ua->prop->list_update) {
+	show_prop_uim_agent_context(ua);
 
-  if (current->prop->label_update) {
-	announce_prop_label_update(current->prop, current->encoding);
-	current->prop->label_update = 0;
+	announce_prop_list_update(ua->prop, ua->encoding);
+	ua->prop->list_update = 0;
+
+	show_im_uim_agent_context(ua);
   }
 }
 
@@ -298,23 +363,24 @@ check_prop_list_label(void)
 static int
 process_command(int serial, int cid, char *cmd)
 {
-  char *opt, *opt2;
+  /*char *opt, *opt2; */
+  char *p, *opt;
   int ret;
 
+  if ((p = strchr(cmd, '\n')))
+	*p = '\0';
+  else
+	return -1;
+
   if ((opt = strchr(cmd, ' '))) {
 	*opt = '\0';
 	opt ++;
-	if ((opt2 = strchr(opt, ' '))) {
-	  *opt2 = '\0';
-	  opt2 ++;
-	} else {
-	  opt2 = NULL;
-	}
   } else {
 	opt = NULL;
-	opt2 = NULL;
   }
 
+  debug_printf(DEBUG_NOTE, "cmd: %s\n", cmd);
+
   if (strcmp(cmd, "RELEASE") == 0)
 	ret = cmd_release(cid);
   else if (strcmp(cmd, "UNFOCUSED") == 0)
@@ -330,28 +396,29 @@ process_command(int serial, int cid, char *cmd)
   else if (strcmp(cmd, "RESET") == 0)
 	ret = cmd_reset(cid);
   else if (strcmp(cmd, "CHANGE") == 0)
-  	ret = cmd_change(cid, opt, opt2);
-  else if (strcmp(cmd, "LIST") == 0)
-  	ret = cmd_list();
-  else if (strcmp(cmd, "SETENC") == 0)
-  	ret = cmd_setenc(opt, opt2);
-  else if (strcmp(cmd, "GETENC") == 0)
-  	ret = cmd_getenc(opt); /* for debug */
+  	ret = cmd_change(cid, opt);
   else if (strcmp(cmd, "PROP") == 0)
   	ret = cmd_prop(cid, opt);
   else if (strcmp(cmd, "LABEL") == 0)
   	ret = cmd_label(cid);
+  else if (strcmp(cmd, "HELPER") == 0)
+	ret = cmd_helper(cid, opt);
+  else if (strcmp(cmd, "NOP") == 0)
+  	ret = cmd_nop(cid);
+  else if (strcmp(cmd, "LIST") == 0)
+  	ret = cmd_list();
+  else if (strcmp(cmd, "SETENC") == 0)
+  	ret = cmd_setenc(opt);
+  else if (strcmp(cmd, "GETENC") == 0)
+  	ret = cmd_getenc(opt); /* for debug */
   else
 	ret = cmd_error();
 
-  check_prop_list_label();
-
-  check_default_engine();
-
   return ret;
 }
 
 
+
 static int
 analyze_keyvector(char *vector, uim_key *ukey, char *keyname)
 {
@@ -478,74 +545,73 @@ analyze_keyvector(char *vector, uim_key *ukey, char *keyname)
 static int
 process_keyvector(int serial, int cid, uim_key ukey, const char *keyname)
 {
-  int ret;
+  int ret, ret2;
 
-  if (current == NULL || 
+  if (! focused ||
+	  current == NULL || 
 	  (current != NULL && current->context_id != cid)) {
 
-	if (focused(get_uim_agent_context(cid)) < 0) {
+	if (set_current_uim_agent_context(get_uim_agent_context(cid)) < 0) {
 	  debug_printf(DEBUG_WARNING, "context %d not found\n", cid);
 	  return -1;
 	}
   }
 
+  focused = 1;
+
 
   if (ukey.key >= 0) {
 	/* key input is received by requested context */
+	debug_printf(DEBUG_NOTE, "uim_press_key\n");
 	ret = uim_press_key(current->context, ukey.key, ukey.mod);
 
-	uim_release_key(current->context, ukey.key, ukey.mod);
+	debug_printf(DEBUG_NOTE, "uim_release_key\n");
+	ret2 = uim_release_key(current->context, ukey.key, ukey.mod);
+
+	debug_printf(DEBUG_NOTE, "ret = %d, ret2 = %d\n", ret, ret2);
+
+	show_commit_string_uim_agent_context(current);
 
 	if (ret > 0) {
 	  /* uim did not process the key */
 
-	  if (current->pe->head == NULL || current->pe->length == 0) {
-		/* no preedit */
+	  if (ukey.mod & UMod_Shift && ukey.key >= 0x41 && ukey.key <= 0x5a)
+		ukey.mod &= ~UMod_Shift;
 
-		if (ukey.mod & UMod_Shift && ukey.key >= 0x41 && ukey.key <= 0x5a)
-		  ukey.mod &= ~UMod_Shift;
+	  if (ukey.mod != 0 || ukey.key > 255) {
 
-		if (ukey.mod != 0 || ukey.key > 255) {
+		a_printf(" ( n [(");
 
-		  a_printf(" ( n [(");
+		if (ukey.mod & UMod_Control) a_printf("control ");
+		if (ukey.mod & UMod_Alt) a_printf("meta ");
+		/* if (ukey->mod & UMod_Shift) a_printf("shift "); */
+		if (ukey.mod & UMod_Hyper) a_printf("hyper ");
+		if (ukey.mod & UMod_Super) a_printf("super ");
 
-		  if (ukey.mod & UMod_Control) a_printf("control ");
-		  if (ukey.mod & UMod_Alt) a_printf("meta ");
-		  /* if (ukey->mod & UMod_Shift) a_printf("shift "); */
-		  if (ukey.mod & UMod_Hyper) a_printf("hyper ");
-		  if (ukey.mod & UMod_Super) a_printf("super ");
+		if (ukey.key > 255)
+		  a_printf("%s", keyname);
+		else
+		  a_printf("%d", ukey.key);
 
-		  if (ukey.key > 255)
-			a_printf("%s", keyname);
-		  else
-			a_printf("%d", ukey.key);
-
-		  a_printf(")] ) ");
-
-		} else {
-		  a_printf(" ( n [%d] ) ", ukey.key);
-		}
+		a_printf(")] ) ");
 
 	  } else {
-		/* discard the key if preedit exists */
-		show_preedit(current->pe);
-		show_candidate(current->pe->cand);
-		/* a_printf(" ( n ) "); */
+		a_printf(" ( n [%d] ) ", ukey.key);
 	  }
 			
-	} else {
-	  /* key has been processed by uim */
-	  show_preedit(current->pe);
-	  show_candidate(current->pe->cand);
 	}
+	show_preedit_uim_agent_context(current);
+	show_candidate_uim_agent_context(current);
   } else {
 	/* ukey.key < 0 */
-	show_preedit(current->pe);
-	show_candidate(current->pe->cand);
-	a_printf(" ( n ) ");
+	show_commit_string_uim_agent_context(current);
+	show_preedit_uim_agent_context(current);
+	show_candidate_uim_agent_context(current);
+	a_printf(" ( n ) "); /* dummy */
   }
 
-  check_prop_list_label();
+  /*show_prop_uim_agent_context(current);*/
+  check_prop_list_update(current);
 
   check_default_engine();
 
@@ -554,55 +620,13 @@ process_keyvector(int serial, int cid, uim_key ukey, const char *keyname)
 
 
 
-static void
-wait_until_data_arrival(fd_set *rfds)
+void
+cleanup(void)
 {
-  int fdmax = STDIN_FILENO;
-
-  if (helper_fd > 0) {
-	FD_ZERO(rfds);
-	FD_SET(helper_fd, rfds);
-	if (helper_fd > fdmax) fdmax = helper_fd;
-  }
-
-  FD_SET(STDIN_FILENO, rfds);
-  
-  if (select(fdmax + 1, rfds, NULL, NULL, NULL) < 0) {
-	debug_printf(DEBUG_ERROR, "select error\n");
-  }
+  uim_quit();
 }
 
 
-static int
-take_one_line_from_buffer(char *buf, char *input)
-{
-  char *p = strchr(buf, '\n'); 
-  int rest; 
-
-  if (!p) {
-	debug_printf(DEBUG_ERROR, "something wrong occured\n");
-	return 0;
-  }
-
-  *p = '\0';
-  strcpy(input, buf);
-
-  rest = strlen(p + 1);
-  memmove(buf, (p + 1), rest);
-
-  return rest;
-}
-
-/*
-static int
-uim_agent_init(void)
-{
-  init_default_engine();
-  return 1;
-}
-*/
-
-
 int
 main(int argc, char *argv[])
 {
@@ -623,49 +647,22 @@ main(int argc, char *argv[])
 	return -1;
   }
 
-  /*  uim_agent_init();*/
+  atexit(cleanup);
+
 
   a_printf("OK\n");
 
   while (1) {
 	int cid, serial;
 	char *p1, *p2, *c;
-	int offset = 0, more_input = 0;
-	char input[256], buf[256], keyname[32];
+	char buf[2048], keyname[32];
 	uim_key ukey;
-	fd_set rfds;
 
 	fflush(stdout);
 
-	buf[sizeof(buf) - 1] = '\0';
+	fgets(buf, sizeof(buf), stdin);
 
-	if (!more_input) {
-	  check_helper_connection();   /* keep helper connection */
-
-	  wait_until_data_arrival(&rfds);
-
-	  if (FD_ISSET(STDIN_FILENO, &rfds)) {
-		/* read from STDIN if input data exists  */
-		read(STDIN_FILENO, buf + offset, sizeof(buf) - offset - 1);
-	  } else if (FD_ISSET(helper_fd, &rfds)) {
-		/* process helper message only when no input data */
-		output_enable = 0;
-		helper_handler();
-		output_enable = 1;
-	  }
-	}
-
-	if ((p1 = strchr(buf, '\n'))) {
-	  offset = take_one_line_from_buffer(buf, input);
-	  more_input = strchr(buf, '\n') ? 1 : 0;
-	} else {
-	  /* wait \n */
-	  offset = strlen(buf);
-	  more_input = 0;
-	  continue;
-	}
-
-	p1 = input;
+	p1 = buf;
 	serial = -1;
 
 	/*
@@ -733,6 +730,7 @@ main(int argc, char *argv[])
 		  a_printf(" ( a ) ");
 
 		a_printf(" )\n");
+		fflush(stdout);
 
 		continue;
 	  }
@@ -743,6 +741,8 @@ main(int argc, char *argv[])
 	} else if (*p1 >= 'A' && *p1 <= 'Z') {
 	  /* command */
 
+	  if (strncmp(p1, "QUIT", 4) == 0) goto QUIT;
+
 	  a_printf("( %d %d ", serial, cid);
 	  if (process_command(serial, cid, p1) < 0) {
 		debug_printf(DEBUG_WARNING, "command error\n");
@@ -751,7 +751,8 @@ main(int argc, char *argv[])
 		a_printf(" ( a ) ");   /* command ok */
 	  }
 
-  a_printf(" )\n");
+	  a_printf(" )\n");
+	  fflush(stdout);
 
 	  continue;
 	}
@@ -760,8 +761,10 @@ main(int argc, char *argv[])
 	
   ERROR:
 	a_printf("( %d 0 ( x ) )\n", serial);
+	fflush(stdout);
   }
 
+ QUIT:
 
   uim_quit();
   return 0;
