@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -35,7 +35,7 @@
  * ץꥨǥåȤ䥹ơ饤褹
  */
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 #ifndef DEBUG
 #define NDEBUG
@@ -438,7 +438,7 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
         }
         goto end_candidate;
       } else if (g_opt.status_type == BACKTICK) {
-        strncpy(s_candbuf, statusline_str, CANDSIZE - 1);
+        strlcpy(s_candbuf, statusline_str, CANDSIZE);
       }
     }
     if (prev_candidate_col != candidate_col) {
@@ -464,7 +464,7 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
           put_uim_str(candidate_str, UPreeditAttr_Reverse);
         } else if (g_opt.status_type == BACKTICK) {
           int byte;
-          strncpy(s_candbuf, statusline_str, CANDSIZE - 1);
+          strlcpy(s_candbuf, statusline_str, CANDSIZE);
           byte = (width2byte(statusline_str, candidate_col))[0] + strlen(candidate_str);
           if (0 <= byte && byte <= CANDSIZE - 1) {
             s_candbuf[byte] = ']';
@@ -566,7 +566,7 @@ end_candidate:
         }
 
       } else if (g_opt.status_type == BACKTICK) {
-        strncpy(s_modebuf, mode_str, sizeof(s_modebuf) - 1);
+        strlcpy(s_modebuf, mode_str, sizeof(s_modebuf));
       }
     }
   }
