@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005 uim Project http://uim.freedesktop.org/
+ *                2005-2006 uim Project http://uim.freedesktop.org/
  *
  *  All rights reserved.
  *
@@ -35,7 +35,7 @@
 #define __dict_cclass_dialog_h_included__
 
 #include <gtk/gtk.h>
-#include "dict-canna.h"
+#include "dict-canna-cclass.h"
 
 gchar *cclass_dialog(gint cclass_type, gint system);
 
