@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004,2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,15 +31,27 @@
 
 */
 
+#include <config.h>
+
 #include <string.h>
 
 #include "dict-dict.h"
 
 /* FIXME! use function */
+#ifdef USE_ANTHY
 extern uim_dict_class uim_dict_class_anthy;
+#endif
+#ifdef USE_CANNA
+extern uim_dict_class uim_dict_class_canna;
+#endif
 
 static uim_dict_class *classes[] = {
+#ifdef USE_ANTHY
   &uim_dict_class_anthy,
+#endif
+#ifdef USE_CANNA
+  &uim_dict_class_canna,
+#endif
 };
 static unsigned int nr_classes = sizeof(classes) / sizeof(uim_dict_class *);
 
@@ -94,20 +106,22 @@ uim_dict_unref(uim_dict *dict)
     uim_dict_close(dict);
 }
 
+/* fail: 0, success: 1 */
 int
 uim_dict_add_word(uim_dict *dict, uim_word *word)
 {
   if (!dict)
-    return -1;
+    return 0;
 
   if (!dict->funcs || !dict->funcs->add_word) {
     /* warning? */
-    return -1;
+    return 0;
   }
 
   return dict->funcs->add_word(dict, word);
 }
 
+/* fail: 0, success: 1 */
 int
 uim_dict_change_word(uim_dict *dict, uim_word *word)
 {
@@ -122,6 +136,7 @@ uim_dict_change_word(uim_dict *dict, uim_word *word)
   return dict->funcs->change_word(dict, word);
 }
 
+/* fail: 0, success: 1 */
 int
 uim_dict_remove_word(uim_dict *dict, uim_word *word)
 {
