@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004,2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,6 +31,8 @@
 
 */
 
+#include <config.h>
+
 #include <gtk/gtk.h>
 
 #include <locale.h>
@@ -39,18 +41,18 @@
 #include <errno.h>
 #include <unistd.h>
 
-#include <uim/uim.h>
-#include <uim/uim-helper.h>
-#include "uim/config.h"
-#include "uim/gettext.h"
+#include "uim.h"
+#include "uim-helper.h"
+#include "gettext.h"
 
 #include "dict-word-win-gtk.h"
 #include "dict-word-list-win-gtk.h"
+#include "dict-word-list-view-gtk.h"
 
 static unsigned int read_tag;
 static int uim_fd;  /* file descriptor to connect helper message bus */
 static int ae_mode; /* add mode or edit mode */
-static int input_method;
+static int g_startup_dictionary;
 
 enum {
   MODE_EDIT,
@@ -58,15 +60,6 @@ enum {
   NR_MODE
 };
 
-/* should be implemeted as loadable module */
-enum {
-  IM_ANTHY,
-  IM_CANNA,
-  IM_SKK,
-  IM_PRIME,
-  NR_IM
-};
-
 static void
 helper_disconnect_cb(void)
 {
@@ -79,7 +72,7 @@ fd_read_cb(GIOChannel *channel, GIOCondition c, gpointer p)
 {
   char *tmp;
   int fd = g_io_channel_unix_get_fd(channel);
-  
+
   uim_helper_read_proc(fd);
   while ((tmp = uim_helper_get_message())) {
     /* parse_helper_str(tmp); */
@@ -118,7 +111,7 @@ parse_arg(int argc, char *argv[])
 
   ae_mode = MODE_EDIT;
 
-  while ((ch = getopt(argc, argv, "aei:")) != -1)
+  while ((ch = getopt(argc, argv, "aehi:")) != -1)
   {
     switch (ch) {
     case 'a':
@@ -129,19 +122,29 @@ parse_arg(int argc, char *argv[])
       break;
     case 'i':
       if (!strcmp(optarg, "anthy"))
-	input_method = IM_ANTHY;
+	g_startup_dictionary = DICT_ENUM_DICTIONARY_TYPE_ANTHY;
       else if (!strcmp(optarg, "canna"))
-	input_method = IM_CANNA;
+	g_startup_dictionary = DICT_ENUM_DICTIONARY_TYPE_CANNA;
       else if (!strcmp(optarg, "prime"))
-	input_method = IM_PRIME;
+	g_startup_dictionary = DICT_ENUM_DICTIONARY_TYPE_PRIME;
       else if (!strcmp(optarg, "skk"))
-	input_method = IM_SKK;
+	g_startup_dictionary = DICT_ENUM_DICTIONARY_TYPE_SKK;
       else
-	input_method = IM_ANTHY;
+	g_startup_dictionary = DICT_ENUM_DICTIONARY_TYPE_ANTHY;
+      break;
+    case 'h':
+      fprintf(stderr, "Usage: uim-dict-gtk [OPTION...]\n");
+      fprintf(stderr, "\n");
+      fprintf(stderr, "Options:\n");
+      fprintf(stderr, " -h            Show this help\n");
+      fprintf(stderr, " -i [IM]       Open a dictionary for IM [anthy, canna]\n");
+      fprintf(stderr, " -e            Start with editing mode (default)\n");
+      fprintf(stderr, " -a            Start with adding mode\n");
+      exit(1);
       break;
     default:
       ae_mode = MODE_EDIT;
-      /* input_method = get_current_im(); */
+      /* g_startup_dictionary = get_current_im(); */
     }
   }
 
@@ -156,13 +159,14 @@ create_window_anthy(void)
   uim_dict *dict;
 
   if (ae_mode == MODE_EDIT) {
-    window = word_list_window_new();
+    window = word_list_window_new(DICT_ENUM_DICTIONARY_TYPE_ANTHY);
+    if (WORD_LIST_VIEW(WORD_LIST_WINDOW(window)->word_list)->dict == NULL)
+      return NULL;
   } else {
     dict = uim_dict_open(N_("Anthy private dictionary"));
     if (!dict)
       return NULL;
     window = word_window_new(WORD_WINDOW_MODE_ADD, dict);
-    uim_dict_unref(dict);
   }
 
   gtk_widget_show(window);
@@ -170,8 +174,33 @@ create_window_anthy(void)
   return window;
 }
 
-static void
-window_destroy_cb(GtkWidget *widget, gpointer data)
+static GtkWidget *
+create_window_canna(void)
+{
+  GtkWidget *window;
+  uim_dict *dict;
+
+  if (ae_mode == MODE_EDIT) {
+    window = word_list_window_new(DICT_ENUM_DICTIONARY_TYPE_CANNA);
+    if (WORD_LIST_VIEW(WORD_LIST_WINDOW(window)->word_list)->dict == NULL)
+      return NULL;
+  } else {
+    dict = uim_dict_open(N_("Canna private dictionary"));
+    if (!dict) {
+	    fprintf(stderr, "uim_dict_open() canna NULL\n");
+      return NULL;
+    }
+    window = word_window_new(WORD_WINDOW_MODE_ADD, dict);
+  }
+
+  gtk_widget_show(window);
+
+  return window;
+}
+
+
+void
+dict_window_destroy_cb(GtkWidget *widget, gpointer data)
 {
   gtk_main_quit();
 }
@@ -181,19 +210,19 @@ create_window(void)
 {
   GtkWidget *window = NULL;
 
-  switch (input_method) {
-  case IM_ANTHY:
+  switch (g_startup_dictionary) {
+  case DICT_ENUM_DICTIONARY_TYPE_ANTHY:
     window = create_window_anthy();
     break;
-  case IM_CANNA:
-    /* window = create_window_canna(); */
+  case DICT_ENUM_DICTIONARY_TYPE_CANNA:
+    window = create_window_canna();
     break;
-  case IM_SKK:
-    /* create_window_skk();*/
-    break;
-  case IM_PRIME:
+  case DICT_ENUM_DICTIONARY_TYPE_PRIME:
     /* create_window_prime();*/
     break;
+  case DICT_ENUM_DICTIONARY_TYPE_SKK:
+    /* create_window_skk();*/
+    break;
   default:
     return -1;
   }
@@ -202,23 +231,42 @@ create_window(void)
     return -1;
 
   g_signal_connect(G_OBJECT(window), "destroy",
-		   G_CALLBACK(window_destroy_cb), NULL);
+		   G_CALLBACK(dict_window_destroy_cb), NULL);
 
   return 0;
 }
 
+static void
+setup_default_icon()
+{
+  GdkPixbuf *pixbuf;
+
+  pixbuf = gdk_pixbuf_new_from_file(UIM_PIXMAPSDIR "/uim-dict.png", NULL);
+  if (pixbuf) {
+    GList *list;
+
+    list = NULL;
+    list = g_list_append(list, pixbuf);
+    gtk_window_set_default_icon_list(list);
+    g_list_free(list);
+    g_object_unref(pixbuf);
+  }
+}
+
 int
 main(int argc, char *argv[])
-{  
+{
   gint result;
+
   setlocale(LC_ALL, "");
   gtk_set_locale();
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
-  bind_textdomain_codeset(PACKAGE, "UTF-8"); 
+  bind_textdomain_codeset(PACKAGE, "UTF-8");
   parse_arg(argc, argv);
 
   gtk_init(&argc, &argv);
+  setup_default_icon();
 
   result = create_window();
 
@@ -229,7 +277,7 @@ main(int argc, char *argv[])
 
   /* connect to uim helper message bus */
   uim_fd = -1;
-  check_helper_connection();  
+  check_helper_connection();
 
   gtk_main ();
 
