@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005 uim Project http://uim.freedesktop.org/
+ *                2005-2006 uim Project http://uim.freedesktop.org/
  *
  *  All rights reserved.
  *
@@ -35,7 +35,7 @@
 #define __WORD_LIST_VIEW_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <gtk/gtkscrolledwindow.h>
