@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005 uim Project http://uim.freedesktop.org/
+ *                2005-2006 uim Project http://uim.freedesktop.org/
  *
  *  All rights reserved.
  *
@@ -49,10 +49,11 @@ typedef enum _uim_word_type {
     WORD_TYPE_ANTHY,
     WORD_TYPE_CANNA,
     WORD_TYPE_SKK,
-    WORD_TYPE_PRIME
+    WORD_TYPE_PRIME,
+    WORD_TYPE_UNKNOWN
 } uim_word_type;
 
-/* 
+/*
  *  - ϥåơ֥뤫ǥǡcharsetphondescʳ
  *    ƤΥС򱣤ۤɤ⤷ʤ
  *    ƤΥǡϴؿͳǥ
@@ -74,9 +75,9 @@ typedef struct _uim_word {
 
     char       *phon;		    /* ɤ */
     char       *desc;	            /*  */
-    char       *cclass_code;	    /* ʻ쥳 or ʻ */
-				    /* FIXME! ʻ쥳ɤʻʬۤɤ? */
-				    /* ߤ(ɤǤϤʤ)ʻ̾()ǼƤ */
+    char       *cclass_code;	    /* ʻ ߤ(ɤǤϤʤ)ʻ̾()
+				       ǼƤ */
+    char       *cclass_native;	    /* native cclass code (like #T01). */
     int	        freq;		    /*  */
 
     /* SKK specific fields */       /* FIXME! ΤѾʬΥ */
@@ -92,9 +93,13 @@ typedef struct _uim_word {
 void      word_append    (uim_word **head, uim_word_type type,
 			  char *charset,
 			  char *phon, char *desc,
-			  char *cclass_code, int freq,
+			  const char *cclass_code,
+			  const char *cclass_native,
+			  int freq,
 			  int okuri, char *annotation);
 void      word_free_list (uim_word *head);
 uim_word *word_last      (uim_word *list);
 
+uim_word_type dict_identifier_to_word_type(char *identifier);
+int dict_identifier_to_support_type(char *identifier);
 #endif /* __dict_word_h_included__ */
