@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -33,7 +33,8 @@
 #ifndef _QT_GETTEXT_H_
 #define _QT_GETTEXT_H_
 
-#include "uim/config.h"
+#include <config.h>
+
 #include "uim/gettext.h"
 
 /* NLS can be disabled through the configure --disable-nls option.  */
