@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
 All rights reserved.
 
@@ -30,6 +30,8 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
+#include <config.h>
+
 #include "immodule-quiminputcontext.h"
 
 #include <qnamespace.h>
@@ -44,11 +46,13 @@ SUCH DAMAGE.
 #include <string.h>
 #include <stdlib.h>
 
-#include "uim/config.h"
-
 #include "immodule-candidatewindow.h"
 #include "immodule-qhelpermanager.h"
 
+#ifdef Q_WS_X11
+#include "immodule-quiminputcontext_compose.h"
+#endif
+
 #define DEFAULT_SEPARATOR_STR "|"
 
 QUimInputContext *focusedInputContext = NULL;
@@ -58,6 +62,9 @@ QPtrList<QUimInputContext> contextList;
 QValueList<UIMInfo> uimInfo;
 
 QUimHelperManager * QUimInputContext::m_HelperManager = 0L;
+#ifdef Q_WS_X11
+DefTree *QUimInputContext::mTreeTop = NULL;
+#endif
 
 // I think that current index-based query API of uim for language and
 // input method name is useless and should be redesigned. I will
@@ -86,6 +93,12 @@ QUimInputContext::QUimInputContext( const char *imname, const char *lang )
     if ( !m_HelperManager )
         m_HelperManager = new QUimHelperManager();
 
+#ifdef Q_WS_X11
+    if ( !mTreeTop )
+        create_compose_tree();
+    mCompose = new Compose( mTreeTop, this );
+#endif
+
     createUimInfo();
 
     // read configuration
@@ -108,6 +121,10 @@ QUimInputContext::~QUimInputContext()
         focusedInputContext = NULL;
         disableFocusedContext = true;
     }
+
+#ifdef Q_WS_X11
+    delete mCompose;
+#endif
 }
 
 uim_context QUimInputContext::createUimContext( const char *imname )
@@ -136,6 +153,11 @@ uim_context QUimInputContext::createUimContext( const char *imname )
 
     uim_set_prop_list_update_cb( uc, QUimHelperManager::update_prop_list_cb );
     uim_set_prop_label_update_cb( uc, QUimHelperManager::update_prop_label_cb );
+
+    uim_set_im_switch_request_cb( uc,
+                                  QUimInputContext::switch_app_global_im_cb,
+                                  QUimInputContext::switch_system_global_im_cb);
+
     uim_prop_list_update( uc );
 
     return uc;
@@ -163,7 +185,7 @@ bool QUimInputContext::filterEvent( const QEvent *event )
         modifier |= UMod_Control;
     if ( keyevent->state() & Qt::AltButton )
         modifier |= UMod_Alt;
-#if defined(_WS_X11_)
+#if defined(Q_WS_X11)
     if ( keyevent->state() & Qt::MetaButton )
         modifier |= UMod_Meta;
 #endif
@@ -178,7 +200,14 @@ bool QUimInputContext::filterEvent( const QEvent *event )
         }
         else
         {
-            key = qkey;
+            if ( keyevent->state() & Qt::ControlButton &&
+                 ( ascii >= 0x01 && ascii <= 0x1a ) )
+                if ( keyevent->state() & Qt::ShiftButton )
+                    key = ascii + 0x40;
+                else
+                    key = ascii + 0x60;
+            else
+                key = qkey;
         }
     }
     else
@@ -198,13 +227,26 @@ bool QUimInputContext::filterEvent( const QEvent *event )
         case Qt::Key_Next: key = UKey_Next; break;
         case Qt::Key_Home: key = UKey_Home; break;
         case Qt::Key_End: key = UKey_End; break;
-        case Qt::Key_Zenkaku_Hankaku: key = UKey_Zenkaku_Hankaku; break;
         case Qt::Key_Multi_key: key = UKey_Multi_key; break;
 #if defined(_WS_X11_)
         case Qt::Key_Mode_switch: key = UKey_Mode_switch; break;
 #endif
-        case Qt::Key_Henkan: key = UKey_Henkan_Mode; break;
+        case Qt::Key_Kanji: key = UKey_Kanji; break;
         case Qt::Key_Muhenkan: key = UKey_Muhenkan; break;
+        case Qt::Key_Henkan: key = UKey_Henkan_Mode; break;
+        case Qt::Key_Romaji: key = UKey_Romaji; break;
+        case Qt::Key_Hiragana: key = UKey_Hiragana; break;
+        case Qt::Key_Katakana: key = UKey_Katakana; break;
+        case Qt::Key_Hiragana_Katakana: key = UKey_Hiragana_Katakana; break;
+        case Qt::Key_Zenkaku: key = UKey_Zenkaku; break;
+        case Qt::Key_Hankaku: key = UKey_Hankaku; break;
+        case Qt::Key_Zenkaku_Hankaku: key = UKey_Zenkaku_Hankaku; break;
+        case Qt::Key_Touroku: key = UKey_Touroku; break;
+        case Qt::Key_Massyo: key = UKey_Massyo; break;
+        case Qt::Key_Kana_Lock: key = UKey_Kana_Lock; break;
+        case Qt::Key_Kana_Shift: key = UKey_Kana_Shift; break;
+        case Qt::Key_Eisu_Shift: key = UKey_Eisu_Shift; break;
+        case Qt::Key_Eisu_toggle: key = UKey_Eisu_toggle; break;
         case Qt::Key_F1: key = UKey_F1; break;
         case Qt::Key_F2: key = UKey_F2; break;
         case Qt::Key_F3: key = UKey_F3; break;
@@ -248,14 +290,24 @@ bool QUimInputContext::filterEvent( const QEvent *event )
     if ( type == QEvent::KeyPress )
     {
         notFiltered = uim_press_key( m_uc, key, modifier );
+#ifdef Q_WS_X11
         if ( notFiltered )
-            return FALSE;
+            return mCompose->handle_qkey( keyevent );
+#else
+        if ( notFiltered )
+	    return FALSE;
+#endif
     }
     else if ( type == QEvent::KeyRelease )
     {
         notFiltered = uim_release_key( m_uc, key, modifier );
+#ifdef Q_WS_X11
+        if ( notFiltered )
+            return mCompose->handle_qkey( keyevent );
+#else
         if ( notFiltered )
             return FALSE;
+#endif
     }
 
     return TRUE;
@@ -308,7 +360,7 @@ QUimInputContext * QUimInputContext::focusedIC()
 }
 
 
-void QUimInputContext::setMicroFocus( int x, int y, int w, int h, QFont *f )
+void QUimInputContext::setMicroFocus( int x, int y, int w, int h, QFont * /* f */)
 {
 #ifdef ENABLE_DEBUG
     // qDebug("IC setMicroFocus (%d, %d), (%d, %d)", x, y, w, h);
@@ -316,9 +368,9 @@ void QUimInputContext::setMicroFocus( int x, int y, int w, int h, QFont *f )
     cwin->layoutWindow( x, y, w, h );
 }
 
-void QUimInputContext::mouseHandler( int x, QEvent::Type type,
-                                     Qt::ButtonState button,
-                                     Qt::ButtonState state )
+void QUimInputContext::mouseHandler( int /* x */, QEvent::Type type,
+                                     Qt::ButtonState /* button */,
+                                     Qt::ButtonState /* state */ )
 {
     switch ( type )
     {
@@ -347,6 +399,9 @@ void QUimInputContext::reset()
     candwinIsActive = FALSE;
     cwin->hide();
     uim_reset_context( m_uc );
+#ifdef Q_WS_X11
+    mCompose->reset();
+#endif
 }
 
 QString QUimInputContext::identifierName()
@@ -451,6 +506,18 @@ void QUimInputContext::cand_deactivate_cb( void *ptr )
     ic->candidateDeactivate();
 }
 
+void QUimInputContext::switch_app_global_im_cb( void *ptr, const char *name )
+{
+    QUimInputContext *ic = ( QUimInputContext* ) ptr;
+    ic->switch_app_global_im( name );
+}
+
+void QUimInputContext::switch_system_global_im_cb( void *ptr, const char *name )
+{
+    QUimInputContext *ic = ( QUimInputContext* ) ptr;
+    ic->switch_system_global_im( name );
+}
+
 void QUimInputContext::commitString( const QString& str )
 {
     if ( !isComposing() )
@@ -620,6 +687,29 @@ void QUimInputContext::candidateDeactivate()
     candwinIsActive = false;
 }
 
+void QUimInputContext::switch_app_global_im( const char *name )
+{
+    QUimInputContext *cc;
+    QString im_name_sym;
+
+    im_name_sym.sprintf( "'%s", name);
+
+    for ( cc = contextList.first(); cc; cc = contextList.next() )
+    {
+        if (cc != this) {
+            uim_switch_im( cc->uimContext(), name );
+            cc->readIMConf();
+        }
+    }
+    uim_prop_update_custom(this->uimContext(), "custom-preserved-default-im-name", ( const char* ) im_name_sym );
+}
+
+void QUimInputContext::switch_system_global_im( const char *name )
+{
+    switch_app_global_im( name );
+    QUimHelperManager::send_im_change_whole_desktop( name );
+}
+
 void QUimInputContext::createUimInfo()
 {
     if ( !uimInfo.isEmpty() )
