@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
 All rights reserved.
 
@@ -54,7 +54,7 @@ public:
     virtual void setHolderWidget( QWidget *w );
 #endif
 
-    virtual bool filterEvent( QEvent *event );
+    virtual bool filterEvent( const QEvent *event );
 
 signals:
     void imEventGenerated( QWidget *, QIMEvent * );
