@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -30,6 +30,8 @@
  SUCH DAMAGE.
 
 */
+#include <config.h>
+
 #include "pref-qt.h"
 #include "pref-customwidgets.h"
 #include "pref-kseparator.h"
@@ -57,7 +59,6 @@
 #include <qmessagebox.h>
 #include <qsettings.h>
 
-#include "uim/config.h"
 #include "qtgettext.h"
 
 #include <stdlib.h>
@@ -74,7 +75,9 @@ UimPrefDialog::UimPrefDialog( QWidget *parent, const char *name )
         checkDotUimFile();        
         setupWidgets();
     } else {
+#if defined(ENABLE_DEBUG)
         qDebug("uim_custom_enable() failed.");
+#endif
         uim_quit();
         QApplication::exit( -1 );
     }
@@ -291,7 +294,9 @@ void UimPrefDialog::slotApply()
     if( !m_isValueChanged )
         return;
 
+#if defined(ENABLE_DEBUG)
     qDebug("start saving....");
+#endif
 
     uim_custom_save();
     uim_custom_broadcast_reload_request();
@@ -365,7 +370,9 @@ void QConfirmDialog::showOnStart( bool isShowOnStart )
     QSettings settings;
     settings.writeEntry( m_confname, isShowOnStart );
 
+#if defined(ENABLE_DEBUG)
     qDebug("show on start = %d", isShowOnStart );
+#endif
 }
 
 
