@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -42,19 +42,23 @@
 #include "qtgettext.h"
 
 static const QString ICONDIR = UIM_PIXMAPSDIR;
+static const QString ACTION_ICONDIR = KDE_ICONDIR "/crystalsvg/32x32/actions";
 
 QUimHelperToolbar::QUimHelperToolbar( QWidget *parent, const char *name, WFlags f )
     : QHBox( parent, name, f )
 {
-    new UimStateIndicator( this );
+    UimStateIndicator *indicator = new UimStateIndicator( this );
+
+    QObject::connect( indicator, SIGNAL( indicatorResized() ), this, SLOT( slotIndicatorResized() ) );
 
     m_swicon = QPixmap( ICONDIR + "/switcher-icon.png" );
-    m_preficon = QPixmap( ICONDIR + "/configure-qt.png");
+    m_preficon = QPixmap( ACTION_ICONDIR + "/configure.png");
+    m_dicticon = QPixmap( ICONDIR + "/uim-dict.png");
 
     m_contextMenu = new QPopupMenu( this );
     m_contextMenu->insertItem( m_swicon,   _("Switch input method"), this, SLOT(slotExecImSwitcher()) );
     m_contextMenu->insertItem( m_preficon, _("Preference"), this, SLOT(slotExecPref()) );
-    m_contextMenu->insertItem( _("Japanese dictionary editor"), this, SLOT(slotExecDict()) );
+    m_contextMenu->insertItem( m_dicticon, _("Japanese dictionary editor"), this, SLOT(slotExecDict()) );
     m_contextMenu->insertItem( _("Input pad"), this, SLOT(slotExecInputPad()) );
     m_contextMenu->insertItem( _("Handwriting input pad"), this, SLOT(slotExecHandwritingInputPad()) );
     m_contextMenu->insertItem( _("Help"), this, SLOT(slotExecHelp()) );
@@ -82,6 +86,11 @@ void QUimHelperToolbar::contextMenuEvent( QContextMenuEvent * e )
     }
 }
 
+void QUimHelperToolbar::slotIndicatorResized()
+{
+    emit toolbarResized();
+}
+
 void QUimHelperToolbar::addExecImSwitcherButton()
 {
     uim_bool isShowSwitcher = uim_scm_symbol_value_bool("toolbar-show-switcher-button?");
@@ -136,7 +145,10 @@ void QUimHelperToolbar::addExecDictButton()
         return;
 
     QToolButton *dictButton = new QHelperToolbarButton( this );
-    dictButton->setText( "Dic" );
+    if( !m_dicticon.isNull() )
+        dictButton->setPixmap( m_dicticon );
+    else
+        dictButton->setText( "Dic" );
 
     QObject::connect( dictButton, SIGNAL( clicked() ),
                       this, SLOT( slotExecDict() ) );
