@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -30,6 +30,8 @@
  SUCH DAMAGE.
 
 */
+#include <config.h>
+
 #include "toolbar-common-uimstateindicator.h"
 
 #include <qsocketnotifier.h>
@@ -42,6 +44,7 @@
 #include <string.h>
 #include <stdlib.h>
 
+static const QString ICONDIR = UIM_PIXMAPSDIR;
 static int uim_fd;
 static QHelperToolbarButton *fallbackButton = NULL;
 static QSocketNotifier *notifier = NULL;
@@ -95,17 +98,18 @@ void UimStateIndicator::parseHelperStr( const QString& str )
     {
         if ( lines[ 0 ] == "prop_list_update" )
             propListUpdate( lines );
-        else if ( lines[ 0 ] == "prop_label_update" )
-            propLabelUpdate( lines );
     }
 }
 
 void UimStateIndicator::propListUpdate( const QStringList& lines )
 {
-    if ( !buttons.isEmpty() )
-        buttons.clear();
-
+    QPtrList<QHelperToolbarButton> tmp_button_list;
+    QHelperToolbarButton *old_button;
     QHelperPopupMenu *popupMenu = NULL;
+    bool size_changed = false;
+
+    tmp_button_list = buttons;
+    old_button = tmp_button_list.first();
 
     QStringList::ConstIterator it = lines.begin();
     const QStringList::ConstIterator end = lines.end();
@@ -122,63 +126,64 @@ void UimStateIndicator::propListUpdate( const QStringList& lines )
                     delete fallbackButton;
                     fallbackButton = NULL;
                 }
-                // create popup
-                popupMenu = new QHelperPopupMenu( 0 );
-                popupMenu->setCheckable( true );
 
                 // create button
-                QHelperToolbarButton *button = new QHelperToolbarButton( this );
-                button->setText( fields[ 1 ] );
-                QToolTip::add( button, fields[ 2 ] );
+                QHelperToolbarButton *button;
+                if (old_button) {
+                    button = old_button;
+                    delete button->popup();
+                } else {
+                    button = new QHelperToolbarButton( this );
+                    buttons.append( button );
+                    size_changed = true;
+                }
+                QPixmap icon = QPixmap(ICONDIR + "/" + fields[1] + ".png");
+                if (!icon.isNull())
+                    button->setPixmap(icon);
+                else
+                    button->setText( fields[ 2 ] );
+                QToolTip::add( button, fields[ 3 ] );
+
+                // create popup
+                popupMenu = new QHelperPopupMenu( button );
+                popupMenu->setCheckable( true );
                 button->setPopup( popupMenu );
                 button->setPopupDelay( 50 );
+
                 button->show();
 
-                buttons.append( button );
+                old_button = tmp_button_list.next();
             }
             else if ( fields[ 0 ].startsWith( "leaf" ) )
             {
                 if ( popupMenu
-                        && !fields[ 2 ].isEmpty()
+                        && !fields[ 1 ].isEmpty()
                         && !fields[ 3 ].isEmpty()
-                        && !fields[ 4 ].isEmpty() )
+                        && !fields[ 4 ].isEmpty()
+                        && !fields[ 5 ].isEmpty() )
                 {
-                    int id = popupMenu->insertHelperItem( fields[ 2 ], fields[ 3 ], fields[ 4 ] );
+                    int id = popupMenu->insertHelperItem( fields[1], fields[ 3 ], fields[ 4 ], fields[ 5 ] );
                     // check the item which is now used
-                    if ( !fields[ 5 ].isEmpty() && fields[ 5 ] == "*" )
+                    if ( !fields[ 6 ].isEmpty() && fields[ 6 ] == "*" )
                         popupMenu->setItemChecked( id, true );
                 }
             }
         }
     }
-}
 
-void UimStateIndicator::propLabelUpdate( const QStringList& lines )
-{
-    unsigned int i = 0;
-    while ( !lines[ i ].isEmpty() )
-        i++;
+    if (old_button)
+        size_changed = true;
 
-    if ( buttons.isEmpty() || buttons.count() != i - 2 )
-    {
-        uim_helper_client_get_prop_list();
-        return ;
+    while (old_button) {
+        QHelperToolbarButton *next;
+
+        next = tmp_button_list.next();
+        buttons.remove(old_button);       
+        old_button = next;
     }
 
-    i = 1;
-    while ( !lines[ i ].isEmpty() )
-    {
-        const QStringList fields = QStringList::split( "\t", lines[ i ] );
-        if ( !fields.isEmpty() && !fields[ 0 ].isEmpty() && !fields[ 1 ].isEmpty() )
-        {
-            // set button label
-            buttons.at( i - 2 ) ->setText( fields[ 0 ] );
-            // set tooltip
-            QToolTip::add( buttons.at( i - 2 ), fields[ 1 ] );
-        }
-
-        i++;
-    }
+    if (size_changed)
+        emit indicatorResized();
 }
 
 void UimStateIndicator::helper_disconnect_cb()
@@ -231,11 +236,18 @@ QHelperPopupMenu::~QHelperPopupMenu()
     msgDict.clear();
 }
 
-int QHelperPopupMenu::insertHelperItem( const QString &menulabelStr,
+int QHelperPopupMenu::insertHelperItem( const QString &indicationIdStr,
+                                        const QString &menulabelStr,
                                         const QString &menutooltipStr,
                                         const QString &menucommandStr )
 {
-    const int id = insertItem( menulabelStr, this, SLOT( slotMenuActivated( int ) ) );
+    int id;
+    QPixmap icon = QPixmap(ICONDIR + "/" + indicationIdStr + ".png");
+
+    if (!icon.isNull())
+        id = insertItem( icon, menulabelStr, this, SLOT( slotMenuActivated( int ) ) );
+    else
+        id = insertItem( menulabelStr, this, SLOT( slotMenuActivated( int ) ) );
 
     setWhatsThis( id, menutooltipStr );
     msgDict.insert( id, new QString( menucommandStr ) );
