@@ -1,6 +1,6 @@
 ;;; custom.scm: Customization support
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -86,8 +86,10 @@
     (string? x)))
 
 (define custom-pathname?
-  (lambda (str)
-    (string? str)))
+  (lambda (str type)
+    (and (string? str)
+	 (symbol? type)
+	 (memq type '(regular-file directory)))))
 
 (define custom-valid-choice?
   (lambda arg
@@ -117,6 +119,10 @@
 			   (custom-exist? key 'key))))
 		key-repls))))
 
+(define custom-pathname-type
+  (lambda (custom-sym)
+    (car (custom-type-attrs custom-sym))))
+
 (define custom-expand-key-references
   (lambda (key)
     (cond
@@ -294,9 +300,7 @@
 ;; TODO: write test
 (define key-list-strip-shift
   (lambda (key-list)
-    (remove (lambda (key)
-	      (eq? key 'Shift_key))
-	    key-list)))
+    (delete 'Shift_key key-list eq?)))
 
 ;; TODO: write test
 (define key-list-strip-regular-shift
@@ -304,9 +308,7 @@
     (let* ((str (find string? key-list))
 	   (printable (string->printable-char str)))
       (if (char-graphic? printable)
-	  (remove (lambda (key)
-		    (eq? key 'Shift_key))
-		  key-list)
+	  (key-list-strip-shift key-list)
 	  key-list))))
 
 ;; TODO: write test
@@ -391,16 +393,12 @@
 ;; API
 (define custom-group-label
   (lambda (gsym)
-    (if (custom-group-rec gsym)
-	(custom-group-rec-label (custom-group-rec gsym))
-	(string-append "Error: " gsym " not found."))))
+    (custom-group-rec-label (custom-group-rec gsym))))
 
 ;; API
 (define custom-group-desc
   (lambda (gsym)
-    (if (custom-group-rec gsym)
-	(custom-group-rec-desc (custom-group-rec gsym))
-	(string-append "Error: " gsym " not found."))))
+    (custom-group-rec-desc (custom-group-rec gsym))))
 
 ;; API
 (define custom-group-subgroups
@@ -481,13 +479,6 @@
 	     (proc))
 	   procs))))
 
-;; TODO: write test
-(define custom-call-all-hook-procs
-  (lambda (hook)
-    (for-each (lambda (pair)
-		((cdr pair)))
-	      hook)))
-
 (define-record 'custom-rec
   '((sym     #f)
     (default #f)
@@ -510,6 +501,14 @@
 			(cdr groups)))
 	   (modified-groups (cons primary-grp subgrps))
 	   (crec (custom-rec-new sym default modified-groups type label desc)))
+      ;; See also require-custom for error handling TODO
+      (for-each (lambda (gsym)
+		  (or (custom-group-rec gsym)
+		      (error (string-append "undefined group '"
+					    (symbol->string gsym)
+					    "' is referred by "
+					    (symbol->string sym)))))
+		modified-groups)
       (set! custom-rec-alist (alist-replace crec custom-rec-alist))
       (custom-call-hook-procs primary-grp custom-group-update-hooks)
       (if (not (symbol-bound? sym))
@@ -647,12 +646,13 @@
   (lambda (sym)
     (let* ((type (custom-type sym))
 	   (attrs (custom-type-attrs sym)))
-      (cond
-       ((or (eq? type 'choice)
-	    (eq? type 'ordered-list))
-	(map custom-choice-rec-sym attrs))
-       (else
-	attrs)))))
+      (case type
+	((choice ordered-list)
+	 (map custom-choice-rec-sym attrs))
+	((integer string)
+	 attrs)
+	(else
+	 ())))))
 
 ;; API
 (define custom-label
@@ -744,10 +744,9 @@
 	 (let ((cb (lambda () (gate-func func ptr custom-sym))))
 	   (custom-add-hook custom-sym hook cb)))))
 
-(define custom-reload-customs
-  (lambda ()
-    (for-each load (reverse custom-required-custom-files))
-    (custom-call-all-hook-procs custom-set-hooks)))
+;;
+;; predefined subgroups
+;;
 
 (define-custom-group 'main
 		     (_ "-")
@@ -757,5 +756,6 @@
 		     (_ "Hidden settings")
 		     (_ "Hidden settings of this group. This group is invisible from uim_custom clients. Exists for internal variable management."))
 
+
 (prealloc-heaps-for-heavy-job)
 (custom-reload-customs)
