@@ -1,6 +1,6 @@
 ;;; generic-key-custom.scm: Customization variables for generic key bindings
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -65,7 +65,7 @@
 	       (_ "[Global] commit")
 	       (_ "long description will be here"))
 
-(define-custom 'generic-cancel-key '("escape" "<IgnoreCase><Control>g")
+(define-custom 'generic-cancel-key '("escape" "<Control>[" "<IgnoreCase><Control>g")
                '(global-keys1)
 	       '(key)
 	       (_ "[Global] cancel")
