@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -66,7 +66,7 @@
 (register-action 'action_generic_off
 		 (lambda (gc)
 		   (list
-		    'figure_generic_off
+		    'off
 		    "-"
 		    (N_ "off")
 		    (N_ "Direct Input Mode")))
@@ -81,7 +81,7 @@
 		   (let* ((im (generic-context-im gc))
 			  (name (symbol->string (im-name im))))
 		     (list
-		      'figure_generic_on
+		      'on
 		      "O"
 		      (N_ "on")
 		      (string-append name (N_ " Mode")))))
