@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -55,19 +55,9 @@
 	    (and (numeral-char? key)
 		 (control-key-mask key-state))))))
 
-;; key
-(define-key prime-language-toggle-key?        "F11")
-(define-key prime-expand-segment-key? '("<Control>o" "<Shift>right"))
-(define-key prime-shrink-segment-key? '("<Control>i" "<Shift>left"))
-
+;; keys without custom
 (define-key prime-escape-key?         '("escape" "<Control>["))
 (define-key prime-space-key?          '(" "))
-(define-key prime-altspace-key?       '("<Control> " "<Alt> "))
-
-(define-key prime-english-next-candidate-key? '("<Control>i" "tab"
-						generic-next-candidate-key?))
-(define-key prime-english-direct-key? '("." "," ":" ";" "(" ")" "\"" "'"
-					"!" "?"))
 
 ;;;; If you're a Vi user, modify the lines below.
 (if prime-custom-app-mode-vi?
@@ -142,24 +132,24 @@
 	  (language (prime-context-language context))
 	  (keymap))
       (cond
-       ((= state 'prime-state-segment)
+       ((eq? state 'prime-state-segment)
 	(set! keymap prime-keymap-segment-state))
 
-       ((= state 'prime-state-modifying)
+       ((eq? state 'prime-state-modifying)
 	(set! keymap prime-keymap-modify-state))
 
-       ((= state 'prime-state-converting)
-	(if (= language 'Japanese)
+       ((eq? state 'prime-state-converting)
+	(if (eq? language 'Japanese)
 	    (set! keymap prime-keymap-conv-state)
 	    (set! keymap prime-keymap-english-conv-state)))
 
-       ((= state 'prime-state-preedit)
-	(if (= language 'Japanese)
+       ((eq? state 'prime-state-preedit)
+	(if (eq? language 'Japanese)
 	    (set! keymap prime-keymap-preedit-state)
 	    (set! keymap prime-keymap-english-preedit-state)))
 
-       ((= state 'prime-state-fund)
-	(if (= language 'Japanese)
+       ((eq? state 'prime-state-fund)
+	(if (eq? language 'Japanese)
 	    (if (prime-context-parent-context context)
 		(set! keymap prime-keymap-child-fund-state)
 		(set! keymap prime-keymap-fund-state))
@@ -353,6 +343,7 @@
     (prime-commit-key?         . prime-command-modify-commit)
     (prime-next-candidate-key? . prime-command-segment-next)
     (prime-prev-candidate-key? . prime-command-segment-prev)
+    (prime-cand-select-key?    . prime-command-segment-select)
     (prime-go-left-edge-key?   . prime-command-modify-cursor-left-edge)
     (prime-go-right-edge-key?  . prime-command-modify-cursor-right-edge)
     (prime-go-left-key?        . prime-command-modify-cursor-left)
@@ -429,8 +420,8 @@
 
 (register-action 'action_prime_mode_latin
 		 (lambda (context)
-		   '(figure_prime_mode_latin
-		     "p[--]"
+		   '(ja_direct
+		     "--"
 		     "̾"
 		     "PRIME򥪥"))
 		 (lambda (context)
@@ -441,8 +432,8 @@
 
 (register-action 'action_prime_mode_hiragana
 		 (lambda (context)
-		   '(figure_prime_mode_hiragana
-		     "P[]"
+		   '(ja_hiragana
+		     ""
 		     "ܸ"
 		     "PRIME򥪥"))
 		 (lambda (context)
@@ -453,8 +444,8 @@
 
 (register-action 'action_prime_mode_wide_latin
 		 (lambda (context)
-		   '(figure_prime_mode_wide_latin
-		     "P[]"
+		   '(ja_fullwidth_alnum
+		     ""
 		     "ѱѿ"
 		     "Ѥ"))
 		 (lambda (context)
@@ -465,8 +456,8 @@
 
 (register-action 'action_prime_mode_application
 		 (lambda (context)
-		   '(figure_prime_mode_application
-		     "P[]"
+		   '(prime_mode_application
+		     ""
 		     "ü"
 		     "ץꥱ¸"))
 		 (lambda (context)
@@ -531,14 +522,14 @@
 ;; This returns context.
 (define prime-context-initialize!
   (lambda (context)
-    (print "prime-context-initialize!")
+    ;(print "prime-context-initialize!")
     (if (null? (prime-context-session context))
 	(begin
 	  ;; The prime server is initialized here.
 	  (prime-lib-init prime-use-unixdomain?)
 	  (let ((session (prime-engine-session-start)))
 	    (prime-custom-init)
-	    (prime-context-set-fund-line!  context (cons (list) (list)))
+	    (prime-context-set-fund-line!  context (cons () ()))
 	    (prime-context-set-session!    context session)
 	    (prime-context-set-lang-session-list!
 	     context
@@ -550,7 +541,7 @@
 ;; uim-contexts and create a new context-data.
 (define prime-context-push
   (lambda (context)
-    (print "prime-context-push")
+    ;(print "prime-context-push")
     (let* ((im (prime-context-im context))
 	   (id (prime-context-id context))
 	   (new-context (prime-context-new2 (prime-context-id context) im)))
@@ -567,7 +558,7 @@
 (define prime-context-pop
   (lambda (context)
     (let ((parent-context (prime-context-parent-context context)))
-      (mapcar
+      (map
        (lambda (lang-pair)
 	 (prime-engine-session-end (cdr lang-pair)))
        (prime-context-lang-session-list context))
@@ -585,7 +576,7 @@
 (define prime-context-history-update!
   (lambda (context)
     (let* ((state          (prime-context-state context))
-	   (selected-index (if (= state 'prime-state-segment)
+	   (selected-index (if (eq? state 'prime-state-segment)
 			       (prime-context-segment-nth context)
 			       (prime-context-nth context))))
       (prime-context-set-history!
@@ -602,7 +593,7 @@
   (lambda (context)
     (let* ((prev-data      (prime-context-history context))
 	   (state          (prime-context-state context))
-	   (selected-index (if (= state 'prime-state-segment)
+	   (selected-index (if (eq? state 'prime-state-segment)
 			       (prime-context-segment-nth context)
 			       (prime-context-nth context))))
       (cond
@@ -657,6 +648,29 @@
     (prime-context-set-nth! context 0)
     ))
 
+(define prime-candidates-get-nth
+  (lambda (context index-no)
+    (if (>= index-no (prime-candidates-get-length context))
+	#f
+	(let ((state (prime-context-state context)))
+	  (if (= state 'prime-state-segment)
+	      (car (nth index-no (prime-context-segment-candidates context)))
+	      (car (nth index-no (prime-context-candidates context))))))))
+
+(define prime-candidates-get-length
+  (lambda (context)
+    (let ((state (prime-context-state context)))
+      (if (= state 'prime-state-segment)
+	  (length (prime-context-segment-candidates context))
+	  (length (prime-context-candidates context))))))
+
+(define prime-candidates-get-index
+  (lambda (context)
+    (let ((state (prime-context-state context)))
+      (if (= state 'prime-state-segment)
+	  (prime-context-segment-nth context)
+	  (prime-context-nth context)))))
+
 (define prime-get-nth-candidate
   (lambda (context n)
     (if (>= n (prime-get-nr-candidates context))
@@ -686,7 +700,7 @@
 
 (define prime-util-assoc-list
   (lambda (lst)
-    (mapcar 
+    (map
      (lambda (str)
        (string-split str "="))
      lst)))
@@ -714,7 +728,7 @@
   (lambda (string)
     (let ((integer 0)
 	  (figure  1))
-      (mapcar
+      (map
        (lambda (digit-string)
 	 (if (string=? digit-string "-")
 	     (set! integer (- integer))
@@ -735,12 +749,15 @@
 ;;;; prime-uim:
 ;;;; ------------------------------------------------------------
 
+;; This returns a pair of the beginning index and the end index of displayed
+;; candidates.
 (define prime-uim-candwin-get-range
   (lambda (context)
-    (let* ((beginning (* (/ (prime-context-nth context) prime-nr-candidate-max)
+    (let* ((beginning (* (/ (prime-candidates-get-index context)
+			    prime-nr-candidate-max)
 			 prime-nr-candidate-max))
 	   (end       (min (+ beginning prime-nr-candidate-max)
-			   (prime-get-nr-candidates context))))
+			   (prime-candidates-get-length context))))
       (cons beginning end))))
 
 ;;;; ------------------------------------------------------------
@@ -778,7 +795,7 @@
     (let* ((result
 	    (prime-engine-send-command (list command prime-session)))
 	   (index (prime-util-string-to-integer (car result)))
-	   (words (mapcar
+	   (words (map
 		   (lambda (string-line)
 		     (let ((word-data (prime-util-string-split string-line
 							       "\t")))
@@ -855,7 +872,7 @@
 
 (define prime-engine-session-language-set
   (lambda (language)
-    (let ((language-string (if (= language 'English) "English" "Japanese")))
+    (let ((language-string (if (eq? language 'English) "English" "Japanese")))
       (car (prime-engine-send-command
 	    (list "session_start" language-string))))))
 
@@ -958,7 +975,7 @@
 	(prime-util-string-split (cdr result) "\t"))
        ((string=? result-type "boolean")
 	(string=? (nth 1 result) "true"))
-       (t
+       (else
 	'unknown))
       )))
 
@@ -990,7 +1007,7 @@
 ;; This changes the typing mode specified by mode-string.
 (define prime-mode-set-mode
   (lambda (context mode-string)
-    (if (= (prime-context-state context) 'prime-state-converting)
+    (if (eq? (prime-context-state context) 'prime-state-converting)
 	(prime-convert-cancel context))
     (prime-engine-edit-set-mode (prime-context-session context) mode-string)))
     
@@ -1021,7 +1038,7 @@
 
 (define prime-command-language-toggle
   (lambda (context key key-state)
-    (let ((next-language (if (= (prime-context-language context) 'English)
+    (let ((next-language (if (eq? (prime-context-language context) 'English)
 			     'Japanese 'English)))
       (prime-mode-language-set context next-language))))
 
@@ -1042,7 +1059,7 @@
 
 (define prime-command-register-mode
   (lambda (context key key-state)
-    (print "prime-command-register-mode")
+    ;(print "prime-command-register-mode")
     (prime-register-mode-on context)))
 
 
@@ -1096,11 +1113,24 @@
     (let* ((nth0 (number->candidate-index (numeral-char->number key)))
 	   (cand-range (prime-uim-candwin-get-range context))
 	   (nth (min (+ (car cand-range) nth0) (cdr cand-range)))
-	   (cand (prime-get-nth-candidate context nth)))
+	   (cand (prime-candidates-get-nth context nth)))
       (if cand
 	  (prime-commit-candidate context nth))
       )))
 
+;; FIXME: Integrate into the above prime-command-conv-select.
+;; FIXME: <Hiroyuki Komatsu> (2005-03-30)
+(define prime-command-segment-select
+  (lambda (context key key-state)
+    (let* ((nth0 (number->candidate-index (numeral-char->number key)))
+	   (cand-range (prime-uim-candwin-get-range context))
+	   (nth (min (+ (car cand-range) nth0) (cdr cand-range)))
+	   (cand (prime-candidates-get-nth context nth)))
+      ;(print cand-range)
+      (if cand
+	  (prime-commit-segment-nth context nth))
+      )))
+
 (define prime-command-conv-input
   (lambda (context key key-state)
     (prime-commit-candidate context (prime-context-nth context))
@@ -1308,19 +1338,19 @@
 (define prime-command-fund-space
   (lambda (context key key-state)
     (cond
-     ((= (prime-context-language context) 'Japanese)
-      (let ((space (if (= prime-custom-japanese-space 'wide) "" " ")))
+     ((eq? (prime-context-language context) 'Japanese)
+      (let ((space (if (eq? prime-custom-japanese-space 'wide) "" " ")))
 	(prime-commit-without-learning context space)))
-     (t
+     (else
       (prime-commit-without-learning context " ")))))
 
 (define prime-command-fund-altspace
   (lambda (context key key-state)
     (cond
-     ((= (prime-context-language context) 'Japanese)
-      (let ((space (if (= prime-custom-japanese-space 'wide) " " "")))
+     ((eq? (prime-context-language context) 'Japanese)
+      (let ((space (if (eq? prime-custom-japanese-space 'wide) " " "")))
 	(prime-commit-without-learning context space)))
-     (t
+     (else
       (prime-commit-without-learning context " ")))))
 
 (define prime-command-fund-commit-ja-direct
@@ -1383,7 +1413,7 @@
 
 (define prime-command-app-mode-start
   (lambda (context key key-state)
-    (print "prime-command-app-mode-start")
+    ;(print "prime-command-app-mode-start")
     (prime-context-set-previous-mode! context (prime-context-mode context))
     (prime-context-set-app-mode-key-list! context
 					  prime-app-mode-end-stroke-list)
@@ -1397,7 +1427,7 @@
 
 (define prime-command-app-mode
   (lambda (context key key-state)
-    (print "prime-command-app-mode")
+    ;(print "prime-command-app-mode")
     (prime-command-app-mode-internal
      context key key-state
      (prime-context-app-mode-key-list context))))
@@ -1407,7 +1437,7 @@
     (let ((key-data (car key-list)))
       (cond
        ;; there's no speficied command then pressed key is passed.
-       ((= key-list '())
+       ((eq? key-list '())
 	(prime-context-set-app-mode-key-list! context
 					      prime-app-mode-end-stroke-list)
 	(prime-commit-raw context))
@@ -1426,7 +1456,7 @@
 	(prime-commit-raw context))
 
        ;; call this command recursively.
-       (t
+       (else
 	(prime-command-app-mode-internal context
 					 key key-state (cdr key-list)))))))
 
@@ -1526,7 +1556,7 @@
 ;; This returns #t if the preediting string is not empty.  Or #f.
 (define prime-preedit-exist?
   (lambda (context)
-    (> (length (prime-preedit-get-string-label context)) 0)))
+    (> (string-length (prime-preedit-get-string-label context)) 0)))
 
 ;; This returns a query string for PRIME server.
 (define prime-preedit-get-string-raw
@@ -1547,6 +1577,8 @@
   (lambda ()
     (let ((typing-method (prime-engine-get-env-typing-method)))
       (cond
+       ((eq? typing-method 'unknown)
+        #f)
        ((string=? typing-method "kana")
 	(prime-dont-use-numeral-key-to-select-cand))
        ((string=? typing-method "tcode")
@@ -1734,11 +1766,12 @@
        ((null? session)
 	#f)  ;; Do nothing.
 
-       (#t
-	;; Store the current preedition into the context
-	(prime-context-set-preedit-line!
-	 context
-	 (prime-engine-edit-get-preedition session))
+       (else
+	(if (not (= (prime-context-mode context) prime-mode-latin))
+	    ;; Store the current preedition into the context
+	    (prime-context-set-preedit-line!
+	     context
+	     (prime-engine-edit-get-preedition session)))
 
 	(prime-update-state context)
 	(prime-update-preedit context)
@@ -1751,7 +1784,7 @@
        ((null? session)
 	#f)  ;; Do nothing.
 
-       (#t
+       (else
 	(prime-update-prediction context)
 	(prime-update-candidate-window context)
 	(prime-update-history context)
@@ -1770,20 +1803,20 @@
   (lambda (context)
     (let ((diff (prime-context-history-compare context)))
       (cond
-       ((= diff 'state)
+       ((eq? diff 'state)
 	(let ((state     (prime-context-state context))
 	      (last-word (prime-context-last-word context)))
 	  (cond
-	   ((= state 'prime-state-preedit)
+	   ((eq? state 'prime-state-preedit)
 	    (prime-convert-get-prediction context))
-	   ((= state 'prime-state-converting)
+	   ((eq? state 'prime-state-converting)
 	    ;; Do nothing.  (prime-convert-get-conversion context) had been
 	    ;; already executed at prime-convert-start-internal
 	    )
-	   ((= state 'prime-state-fund)
+	   ((eq? state 'prime-state-fund)
 	    (prime-context-set-candidates! context '()))
 	    )))
-       ((= diff 'preedit)
+       ((eq? diff 'preedit)
 	(prime-convert-get-prediction context))
        ))))
 
@@ -1791,13 +1824,13 @@
   (lambda (context)
     (let ((diff (prime-context-history-compare context)))
       (cond
-       ((= diff 'state)
+       ((eq? diff 'state)
 	(let ((state (prime-context-state context)))
 	  (cond
-	   ((= state 'prime-state-fund)
+	   ((eq? state 'prime-state-fund)
 	    (im-deactivate-candidate-selector context))
 
-	   ((= state 'prime-state-preedit)
+	   ((eq? state 'prime-state-preedit)
 	    (if (> (prime-get-nr-candidates context) 0)
 		(im-activate-candidate-selector
 		 context
@@ -1805,15 +1838,15 @@
 		 3)))
 ;		 prime-nr-candidate-max)))
 
-	   ((= state 'prime-state-converting)
+	   ((eq? state 'prime-state-converting)
  	    (im-activate-candidate-selector
  	     context (prime-get-nr-candidates context) prime-nr-candidate-max)
 	    (im-select-candidate context (prime-context-nth context)))
 
-	   ((= state 'prime-state-modifying)
+	   ((eq? state 'prime-state-modifying)
 	    (im-deactivate-candidate-selector context))
 
-	   ((= state 'prime-state-segment)
+	   ((eq? state 'prime-state-segment)
  	    (im-activate-candidate-selector
 	     context
 	     (prime-segment-get-candidates-length context)
@@ -1821,12 +1854,12 @@
 	    (im-select-candidate context (prime-context-segment-nth context)))
 	    )))
 
-       ((= diff 'nth)
-	(if (= (prime-context-state context) 'prime-state-segment)
+       ((eq? diff 'nth)
+	(if (eq? (prime-context-state context) 'prime-state-segment)
 	    (im-select-candidate context (prime-context-segment-nth context))
 	    (im-select-candidate context (prime-context-nth context))))
 
-       ((= diff 'preedit)
+       ((eq? diff 'preedit)
 	(if (> (prime-get-nr-candidates context) 0)
 	    (im-activate-candidate-selector
 	     context (prime-get-nr-candidates context) prime-nr-candidate-max)
@@ -1859,12 +1892,12 @@
   (lambda (context)
     (let* ((state (prime-context-state context)))
       (cond
-       ((= state 'prime-state-converting)
+       ((eq? state 'prime-state-converting)
 	(list (cons 'converting (prime-get-current-candidate context))
 	      (cons 'cursor     "")))
 
-       ((or (= state 'prime-state-modifying)
-	    (= state 'prime-state-segment))
+       ((or (eq? state 'prime-state-modifying)
+	    (eq? state 'prime-state-segment))
 	(let* ((line (prime-context-modification context)))
 	  (list (cons 'segment           (nth 0 line))
 		(cons 'segment-highlight (nth 1 line))
@@ -1926,7 +1959,7 @@
 
 (define prime-register-mode-on
   (lambda (context)
-    (print "prime-register-mode-on")
+    ;(print "prime-register-mode-on")
     (let* ((reading (prime-preedit-get-string-label context))
 	   ;; Header and footer strings for a preedition line.
 	   (current-display-head (prime-context-display-head context))
@@ -1965,7 +1998,7 @@
 
 (define prime-release-handler
   (lambda (context)
-    (print "prime-release-handler")
+    ;(print "prime-release-handler")
     (let ((session (prime-context-session context)))
       (if session
 	  (prime-engine-session-end session)))
@@ -1982,7 +2015,7 @@
 
 (define prime-release-key-handler
   (lambda (context key key-state)
-    (print "prime-release-key-handler")
+    ;(print "prime-release-key-handler")
     (if (or (control-char? key)
 	    (= (prime-context-mode context)
 	       prime-mode-latin))
@@ -1994,7 +2027,7 @@
 
 (define prime-reset-handler
   (lambda (context)
-    (print "prime-reset-handler")
+    ;(print "prime-reset-handler")
     ))
 
 (define prime-mode-set
@@ -2003,6 +2036,9 @@
     ;; FIXME: I don't wanna use prime-context-session here.
     ;; FIXME: (2005-02-25) <Hiroyuki Komatsu>
     ;; If the session is #f, the PRIME mode has never been turned on.
+    (if (not (prime-context-session context))
+        (prime-context-initialize! context))
+
     (if (prime-context-session context)
 	(begin
 	  (prime-preedit-reset! context)
@@ -2026,7 +2062,7 @@
 (define prime-get-candidate-handler
   (lambda (context index-no accel-enum-hint)
     (let ((candidate
-	   (if (= (prime-context-state context) 'prime-state-segment)
+	   (if (eq? (prime-context-state context) 'prime-state-segment)
 	       (nth index-no (prime-context-segment-candidates context))
 	       (nth index-no (prime-context-candidates context)))))
       ;; The return value is a list with a candidate string and the next index.
@@ -2042,18 +2078,18 @@
 	  (state      (prime-context-state context)))
       (if (and prime-custom-display-form?
 	       form
-	       (or (= state 'prime-state-converting)
-		   (= state 'prime-state-segment)))
+	       (or (eq? state 'prime-state-converting)
+		   (eq? state 'prime-state-segment)))
 	  (set! string (string-append string "  (" form ")")))
       (if (and prime-custom-display-usage?
 	       usage
-	       (or (= state 'prime-state-converting)
-		   (= state 'prime-state-segment)))
+	       (or (eq? state 'prime-state-converting)
+		   (eq? state 'prime-state-segment)))
 	  (set! string (string-append string "\t" usage)))
       (if (and prime-custom-display-comment?
 	       comment
-	       (or (= state 'prime-state-converting)
-		   (= state 'prime-state-segment)))
+	       (or (eq? state 'prime-state-converting)
+		   (eq? state 'prime-state-segment)))
 	  (set! string (string-append string "\t<" comment ">")))
       string)))
 
@@ -2067,10 +2103,10 @@
 
 (define prime-set-candidate-index-handler
   (lambda (context selection-index)
-    (print "prime-set-candidate-index-handler")
+    ;(print "prime-set-candidate-index-handler")
     (if (prime-context-session context)
 	(begin
-	  (if (= (prime-context-state context) 'prime-state-segment)
+	  (if (eq? (prime-context-state context) 'prime-state-segment)
 	      (prime-commit-segment-nth context selection-index)
 	      (prime-commit-candidate context selection-index))
 	  (prime-update context)
