@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -30,7 +30,11 @@
   SUCH DAMAGE.
 */
 /**/
+#include <config.h>
+
 #include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
 #include <anthy/anthy.h>
 
 #include "uim-scm.h"
@@ -39,6 +43,10 @@
 
 #define MAX_CONTEXT 256
 
+/* handle anthy's version scheme like 7100b */
+static char *anthy_version_major;
+static char anthy_version_minor[2];
+
 static struct context {
   anthy_context_t ac;
 } *context_slot;
@@ -52,24 +60,60 @@ get_anthy_context(int id)
   return context_slot[id].ac;
 }
 
+static uim_lisp
+anthy_version()
+{
+  return uim_scm_cons(uim_scm_make_str(anthy_version_major),
+		      uim_scm_make_str(anthy_version_minor));
+}
+
+static void
+get_anthy_version()
+{
+  const char *str;
+
+  free(anthy_version_major);
+
+  str = anthy_get_version_string();
+
+  if (!str || (!strcmp(str, "(unknown)"))) {
+    anthy_version_major = strdup("-1");
+    anthy_version_minor[0] = '\0';
+  } else {
+    int len;
+
+    len = strlen(str);
+    if (isalpha(str[len - 1])) {
+      anthy_version_major = malloc(len);
+      strlcpy(anthy_version_major, str, len);
+      strlcpy(anthy_version_minor, &str[len - 1], sizeof(anthy_version_minor));
+    } else {
+      anthy_version_major = strdup(str);
+      anthy_version_minor[0] = '\0';
+    }
+  }
+}
+
 static uim_lisp
 init_anthy_lib(void)
 {
   int i;
-  if (context_slot) {
+
+  if (context_slot)
     return uim_scm_t();
-  }
-  if (anthy_init() == -1) {
+
+  if (anthy_init() == -1)
     return uim_scm_f();
-  }
-  context_slot = malloc(sizeof(struct context) *
-			MAX_CONTEXT);
-  if (!context_slot) {
+
+  get_anthy_version();
+
+  context_slot = malloc(sizeof(struct context) * MAX_CONTEXT);
+  if (!context_slot)
     return uim_scm_f();
-  }
-  for (i = 0; i < MAX_CONTEXT; i++) {
+
+  for (i = 0; i < MAX_CONTEXT; i++)
     context_slot[i].ac = NULL;
-  }
+
   return uim_scm_t();
 }
 
@@ -159,27 +203,37 @@ get_nr_candidates(uim_lisp id_, uim_lisp nth_)
 static uim_lisp
 get_nth_candidate(uim_lisp id_, uim_lisp seg_, uim_lisp nth_)
 {
-  int id = uim_scm_c_int(id_);
-  int seg = uim_scm_c_int(seg_);
-  int nth  = uim_scm_c_int(nth_);
-  int buflen;
+  int id, seg, nth, buflen;
   char *buf;
   uim_lisp buf_;
-  anthy_context_t ac = get_anthy_context(id);
-  if (!ac) {
+  anthy_context_t ac;
+  
+  id = uim_scm_c_int(id_);
+  seg = uim_scm_c_int(seg_);
+  nth  = uim_scm_c_int(nth_);
+  ac = get_anthy_context(id);
+
+  if (!ac)
     return uim_scm_f();
-  }
+
   buflen = anthy_get_segment(ac, seg, nth, NULL, 0);
-  if (buflen == -1) {
+  if (buflen == -1)
     return uim_scm_f();
-  }
-  buf = malloc(buflen+1);
-  anthy_get_segment(ac, seg, nth, buf, buflen+1);
+
+  buf = malloc(buflen + 1);
+  anthy_get_segment(ac, seg, nth, buf, buflen + 1);
   buf_ = uim_scm_make_str(buf);
   free(buf);
+
   return buf_;
 }
 
+static uim_lisp
+get_unconv_candidate(uim_lisp id_, uim_lisp seg_)
+{
+  return get_nth_candidate(id_, seg_, uim_scm_make_int(NTH_UNCONVERTED_CANDIDATE));
+}
+
 static uim_lisp
 get_segment_length(uim_lisp id_, uim_lisp nth_)
 {
@@ -223,10 +277,10 @@ commit_segment(uim_lisp id_, uim_lisp s_, uim_lisp nth_)
   return uim_scm_f();
 }
 
-#ifdef HAS_ANTHY_PREDICTION
 static uim_lisp
 set_prediction_src_string(uim_lisp id_, uim_lisp str_)
 {
+#ifdef HAS_ANTHY_PREDICTION
   int id = uim_scm_c_int(id_);
   const char *str = uim_scm_refer_c_str(str_);
   anthy_context_t ac = get_anthy_context(id);
@@ -235,11 +289,15 @@ set_prediction_src_string(uim_lisp id_, uim_lisp str_)
   }
   anthy_set_prediction_string(ac, str);
   return uim_scm_f();
+#else
+  return uim_scm_f();
+#endif
 }
 
 static uim_lisp
 get_nr_predictions(uim_lisp id_)
 {
+#ifdef HAS_ANTHY_PREDICTION
   int id = uim_scm_c_int(id_);
   anthy_context_t ac = get_anthy_context(id);
   struct anthy_prediction_stat ps;
@@ -248,11 +306,15 @@ get_nr_predictions(uim_lisp id_)
   }
   anthy_get_prediction_stat(ac, &ps);
   return uim_scm_make_int(ps.nr_prediction);
+#else
+  return uim_scm_f();
+#endif
 }
 
 static uim_lisp
 get_nth_prediction(uim_lisp id_, uim_lisp nth_)
 {
+#ifdef HAS_ANTHY_PREDICTION
   int id  = uim_scm_c_int(id_);
   int nth = uim_scm_c_int(nth_); 
   int buflen;
@@ -271,11 +333,36 @@ get_nth_prediction(uim_lisp id_, uim_lisp nth_)
   buf_ = uim_scm_make_str(buf);
   free(buf);
   return buf_;
+#else
+  return uim_scm_f();
+#endif
 }
-#endif /* HAS_ANTHY_PREDICTION */
 
+static uim_lisp
+commit_nth_prediction(uim_lisp id_, uim_lisp nth_)
+{
+#ifdef HAS_ANTHY_COMMIT_PREDICTION
+  int id  = uim_scm_c_int(id_);
+  int nth = uim_scm_c_int(nth_); 
+  anthy_context_t ac = get_anthy_context(id);
+
+  if (!ac) {
+    return uim_scm_f();
+  }
+
+  return anthy_commit_prediction(ac, nth) ? uim_scm_f() : uim_scm_t();
+#else
+  return uim_scm_f();
+#endif
+}
+
+#ifndef ENABLE_ANTHY_STATIC
 void
 uim_plugin_instance_init(void)
+#else
+void
+uim_anthy_plugin_instance_init(void)
+#endif
 {
   uim_scm_init_subr_0("anthy-lib-init", init_anthy_lib);
   uim_scm_init_subr_0("anthy-lib-alloc-context", create_context);
@@ -284,18 +371,25 @@ uim_plugin_instance_init(void)
   uim_scm_init_subr_1("anthy-lib-get-nr-segments",get_nr_segments);
   uim_scm_init_subr_2("anthy-lib-get-nr-candidates", get_nr_candidates);
   uim_scm_init_subr_3("anthy-lib-get-nth-candidate", get_nth_candidate);
+  uim_scm_init_subr_2("anthy-lib-get-unconv-candidate", get_unconv_candidate);
   uim_scm_init_subr_2("anthy-lib-get-segment-length", get_segment_length);
   uim_scm_init_subr_3("anthy-lib-resize-segment", resize_segment);
   uim_scm_init_subr_3("anthy-lib-commit-segment", commit_segment);
-#ifdef HAS_ANTHY_PREDICTION
+  uim_scm_init_subr_0("anthy-lib-get-anthy-version", anthy_version);
   uim_scm_init_subr_2("anthy-lib-set-prediction-src-string", set_prediction_src_string);
   uim_scm_init_subr_1("anthy-lib-get-nr-predictions", get_nr_predictions);
   uim_scm_init_subr_2("anthy-lib-get-nth-prediction", get_nth_prediction);
-#endif /* HAS_ANTHY_PREDICTION */
+  uim_scm_init_subr_2("anthy-lib-commit-nth-prediction",
+		      commit_nth_prediction);
 }
 
+#ifndef ENABLE_ANTHY_STATIC
 void
 uim_plugin_instance_quit(void)
+#else
+void
+uim_anthy_plugin_instance_quit(void)
+#endif
 {
   int i;
 
