@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,17 +31,20 @@
 
 */
 
-#include "config.h"
+#include <config.h>
+
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <m17n.h>
+#include "gettext.h"
 #include "uim-scm.h"
 #include "uim-util.h"
 #include "plugin.h"
 
 static int m17nlib_ok;
 static MConverter *converter;
-static char buffer_for_converter[1024]; /* Currently, if preedit strings or
+static char buffer_for_converter[4096]; /* Currently, if preedit strings or
 					   candidate strings over this buffer
 					   size, they will simply ignore. */
 
@@ -52,7 +55,7 @@ static struct im_ {
   MInputMethod *im;
 } *im_array;
 
-static int max_input_contexts;
+static int nr_input_contexts;
 static struct ic_ {
   MInputContext *mic;
   char **old_candidates; /* FIXME: ugly hack for perfomance... */
@@ -60,43 +63,53 @@ static struct ic_ {
   int  nr_candidates;
 } *ic_array;
 
+/* Utility function */
 static char *
-m17nlib_utf8_find_next_char(const char *p);
+m17nlib_utf8_find_next_char(const char *p)
+{
+  if (*p) {
+    for (++p; (*p & 0xc0) == 0x80; ++p)
+      ;
+  }
+  return (char *)p;
+}
 
 static int
 unused_ic_id(void)
 {
   int i;
-  for (i = 0; i < max_input_contexts; i++) {
-    if (!ic_array[i].mic) {
+
+  for (i = 0; i < nr_input_contexts; i++) {
+    if (!ic_array[i].mic)
       return i;
-    }
   }
-  ic_array = realloc(ic_array, sizeof(struct ic_) * (max_input_contexts+1));
-  ic_array[max_input_contexts].mic = NULL;
-  max_input_contexts++;
-  return max_input_contexts - 1;
+
+  ic_array = realloc(ic_array, sizeof(struct ic_) * (nr_input_contexts + 1));
+  ic_array[nr_input_contexts].mic = NULL;
+  nr_input_contexts++;
+
+  return nr_input_contexts - 1;
 }
 
 static void
-pushback_input_method(MInputMethod *im,
-		      char *lib_lang, char *name)
+pushback_input_method(MInputMethod *im, char *lib_lang, char *name)
 {
-  char *lang = uim_get_language_code_from_language_name(lib_lang);
+  const char *lang;
+  
+  if (!strcmp(lib_lang, "t"))
+    lang = "";
+  else
+    lang = lib_lang;
 
-  im_array = realloc(im_array, 
-		     sizeof(struct im_) * (nr_input_methods + 1));
-
-  if (lang != NULL) {
-    im_array[nr_input_methods].lang = strdup(lang);
-  } else {
-    im_array[nr_input_methods].lang = NULL;
-  }
-  im_array[nr_input_methods].name = strdup(name);
+  im_array = realloc(im_array, sizeof(struct im_) * (nr_input_methods + 1));
   im_array[nr_input_methods].im = im;
+  im_array[nr_input_methods].name = strdup(name);
+  im_array[nr_input_methods].lang = strdup(lang);
+
   nr_input_methods++;
 }
 
+#if 0
 static void
 preedit_start_cb(MInputContext *ic, MSymbol command)
 {
@@ -165,43 +178,49 @@ register_callbacks(void)
   mplist_add(minput_default_driver.callback_list, Minput_candidates_draw,  (void *)candidates_draw_cb);
   mplist_add(minput_default_driver.callback_list, Minput_candidates_done,  (void *)candidates_done_cb);*/
 }
+#endif
 
 static uim_lisp
 init_m17nlib()
 {
   MPlist *imlist, *elm;
-  MSymbol utf8 = msymbol("utf8");
+
   M17N_INIT();
   nr_input_methods = 0;
+  nr_input_contexts = 0;
   im_array = NULL;
-  max_input_contexts = 0;
   ic_array = NULL;
 
   imlist = mdatabase_list(msymbol("input-method"), Mnil, Mnil, Mnil);
+
   if (!imlist) {
     /* maybe user forgot to install m17n-db */
     return uim_scm_f();
   }
+
   for (elm = imlist; mplist_key(elm) != Mnil; elm = mplist_next(elm)) {
     MDatabase *mdb = mplist_value(elm);
-    MSymbol *tag = mdatabase_tag(mdb);
+    MSymbol *tag = mdatabase_tag(mdb); /* tag[1]: lang, tag[2]: name */
+
     if (tag[1] != Mnil) {
       MInputMethod *im = minput_open_im(tag[1], tag[2], NULL);
-      if (im) {
-	MSymbol lang = msymbol_get(im->language, Mlanguage);
-	pushback_input_method(im, msymbol_name(lang),
+
+      if (im)
+	pushback_input_method(im, msymbol_name(im->language),
 			      msymbol_name(im->name));
-	
-      }
     }
   }
-  /*  register_callbacks();*/
+#if 0
+  register_callbacks();
+#endif
   m17n_object_unref(imlist);
-  converter = mconv_buffer_converter(utf8, NULL, 0);
-  if (!converter) {
+  converter = mconv_buffer_converter(msymbol("utf8"), NULL, 0);
+
+  if (!converter)
     return uim_scm_f();
-  }
+
   m17nlib_ok = 1;
+
   return uim_scm_t();
 }
 
@@ -212,184 +231,195 @@ convert_mtext2str(MText *mtext)
 		      sizeof(buffer_for_converter));
   mconv_encode(converter, mtext);
   buffer_for_converter[converter->nbytes] = 0;
+
   return strdup(buffer_for_converter);
 }
 
 static uim_lisp
 compose_modep(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  /* range check of id might need. */
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  int id;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_f();
-  }
-  if (ic->candidate_from == ic->candidate_to ||
-     ic->candidate_from > ic->candidate_to) {
+
+  if (ic->candidate_from == ic->candidate_to
+      || ic->candidate_from > ic->candidate_to)
     return uim_scm_f();
-  } else {
+  else
     return uim_scm_t();
-  }
 }
 
 static uim_lisp
 preedit_changedp(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  /* range check of id might need. */
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  int id;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_f();
-  }
-  if (ic->preedit_changed == 1) {
+
+  if (ic->preedit_changed == 1)
     return uim_scm_t();
-  } else {
+  else
     return uim_scm_f();
-  }
 }
 
 static uim_lisp
 get_left_of_cursor(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  int buflen;
-  int i;
+  int id, buflen, i;
   uim_lisp buf_;
-  char *buf;
-  char *p;
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  char *buf, *p;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_make_str("");
-  }
-  if (ic->cursor_pos == 0) {
+
+  if (ic->cursor_pos == 0)
     return uim_scm_make_str("");
-  }
+
   buf = convert_mtext2str(ic->preedit);
-  p = (char *)buf;
+  p = buf;
 
-  for (i=0; i<ic->cursor_pos ;i++) {
+  for (i = 0; i < ic->cursor_pos ;i++)
     p = m17nlib_utf8_find_next_char(p);
-  }
-  *p = 0;
+  *p = '\0';
+
+  buflen = strlen(buf);
+  buf_ = uim_scm_make_str(buf);
+  free(buf);
 
-  buflen = strlen((char *)buf);
-  buf_ = uim_scm_make_str((char *)buf);
   return buf_;
 }
 
 static uim_lisp
 get_right_of_cursor(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  int buflen;
-  int i;
+  int id, buflen, i;
   uim_lisp buf_;
-  char *buf;
-  char *p;
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  char *buf, *p;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_make_str("");
-  }
 
   buf = convert_mtext2str(ic->preedit);
   p = buf;
 
-  for (i=0; i<ic->cursor_pos ;i++) {
+  for (i = 0; i < ic->cursor_pos ;i++)
     p = m17nlib_utf8_find_next_char(p);
-  }
-  buflen = strlen((char *)p);
-  buf_ = uim_scm_make_str((char *)p);
+
+  buflen = strlen(p);
+  buf_ = uim_scm_make_str(p);
+  free(buf);
+
   return buf_;
 }
 
 static uim_lisp
 get_left_of_candidate(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  int buflen;
-  int i;
+  int id, buflen, i;
   uim_lisp buf_;
-  char *buf;
-  char *p;
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  char *buf, *p;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_make_str("");
-  }
-  if (ic->candidate_from == 0) {
+
+  if (ic->candidate_from == 0)
     return uim_scm_make_str("");
-  }
+
   buf = convert_mtext2str(ic->preedit);
   p = buf;
 
-  for (i=0; i<ic->candidate_from ;i++) {
+  for (i = 0; i < ic->candidate_from ;i++)
     p = m17nlib_utf8_find_next_char(p);
-  }
-  *p = 0;
-  buflen = strlen((char *)buf);
-  buf_ = uim_scm_make_str((char *)buf);
+  *p = '\0';
+
+  buflen = strlen(buf);
+  buf_ = uim_scm_make_str(buf);
   free(buf);
+
   return buf_;
 }
 
 static uim_lisp
 get_selected_candidate(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  int buflen;
-  int i;
+  int id, buflen, i;
   uim_lisp buf_;
-  char *buf;
-  char *p;
-  char *start;
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  char *buf, *p, *start;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_make_str("");
-  }
+
   buf = convert_mtext2str(ic->preedit);
   p = buf;
 
-  if (!p) {
+  if (!p)
     return uim_scm_make_str("");
-  }
 
-  for (i=0; i<ic->candidate_from ;i++) {
+  for (i = 0; i < ic->candidate_from ;i++)
     p = m17nlib_utf8_find_next_char(p);
-  }
   start = p;
 
-  for (i=0; i<ic->candidate_to - ic->candidate_from ;i++) {
+  for (i = 0; i < ic->candidate_to - ic->candidate_from ;i++)
     p = m17nlib_utf8_find_next_char(p);
-  }
-  *p = 0;
+  *p = '\0';
 
   buflen = strlen(start);
   buf_ = uim_scm_make_str(start);
   free(buf);
+
   return buf_;
 }
 
 static uim_lisp
 get_right_of_candidate(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  int buflen;
-  int i;
+  int id, buflen, i;
   uim_lisp buf_;
-  char *buf;
-  char *p;
-  MInputContext *ic = ic_array[id].mic;
-  if (!ic) {
+  char *buf, *p;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (!ic)
     return uim_scm_make_str("");
-  }
+
   buf = convert_mtext2str(ic->preedit);
   p = buf;
 
-  for (i=0; i<ic->candidate_to ;i++) {
+  for (i = 0; i < ic->candidate_to ;i++)
     p = m17nlib_utf8_find_next_char(p);
-  }
+
   buflen = strlen(p);
   buf_ = uim_scm_make_str(p);
   free(buf);
+
   return buf_;
 }
 
@@ -403,16 +433,18 @@ static uim_lisp
 get_input_method_name(uim_lisp nth_)
 {
   int nth = uim_scm_c_int(nth_);
+
   if (nth < nr_input_methods) {
     char *name = alloca(strlen(im_array[nth].name) + 20);
 
-    if (im_array[nth].lang != NULL)
-      sprintf(name, "m17n-%s-%s", im_array[nth].lang, im_array[nth].name);
-    else
+    if (!strcmp(im_array[nth].lang, ""))
       sprintf(name, "m17n-%s", im_array[nth].name);
+    else
+      sprintf(name, "m17n-%s-%s", im_array[nth].lang, im_array[nth].name);
 
     return uim_scm_make_str(name);
   }
+
   return uim_scm_f();
 }
 
@@ -420,51 +452,116 @@ static uim_lisp
 get_input_method_lang(uim_lisp nth_)
 {
   int nth = uim_scm_c_int(nth_);
-  if (nth < nr_input_methods) {
-    char *lang = im_array[nth].lang;
-    if (lang != NULL) {
-      return uim_scm_make_str(lang);
-    } else {
-      return uim_scm_make_str("unknown");
-    }
-  }
+
+  if (nth < nr_input_methods)
+    return uim_scm_make_str(im_array[nth].lang);
+
   return uim_scm_f();
 }
 
+static uim_lisp
+get_input_method_short_desc(uim_lisp nth_)
+{
+  int nth;
+  char *str = NULL, *p;
+  uim_lisp ret;
+
+  nth = uim_scm_c_int(nth_);
+
+  if (nth < nr_input_methods) {
+    MInputMethod *im;
+    MText *desc;
+
+    im = im_array[nth].im;
+    desc = minput_get_description(im->language, im->name);
+    if (desc) {
+      int i, len;
+
+      str = convert_mtext2str(desc);
+      p = strchr(str, '.');
+      if (p)
+	*p = '\0';
+      len = strlen(str);
+
+      /*
+       * Workaround for the descriptions which lack period.
+       * Also we avoid the description with non English words.
+       * See https://bugs.freedesktop.org/show_bug.cgi?id=6972
+       */
+      for (i = 0; i < len; i++) {
+	if (str[i] == '\n') {
+	  str[i] = '\0';
+	  break;
+	}
+#ifdef HAVE_ISASCII
+	else if (!isascii((int)str[i])) {
+#else
+	else if ((int)str[i] & ~0x7f) {
+#endif
+	  free(str);
+	  str = NULL;
+	  break;
+	}
+      }
+      m17n_object_unref(desc);
+    }
+
+    if (str) {
+      ret = uim_scm_make_str(str);
+      free(str);
+    } else {
+      ret = uim_scm_make_str(N_("An input method provided by the m17n library"));
+    }
+  } else
+    ret = uim_scm_f();
+
+  return ret;
+}
+
 static MInputMethod *
-find_im_by_name(char *name)
+find_im_by_name(const char *name)
 {
   int i;
-  if (strncmp(name, "m17n-", 5) != 0) {
+  const char *im_name;
+
+  if (strncmp(name, "m17n-", 5) != 0)
     return NULL;
-  }
-  name = &name[5];
+
+  im_name = &name[5];
+
   for (i = 0; i < nr_input_methods; i++) {
     char buf[100];
-    if (im_array[i].lang == NULL) {
+
+    if (!strcmp(im_array[i].lang, ""))
       snprintf(buf, 100, "%s", im_array[i].name);
-    } else {
+    else
       snprintf(buf, 100, "%s-%s", im_array[i].lang, im_array[i].name);
-    }
-    if (!strcmp(name, buf)) {
+
+    if (!strcmp(im_name, buf))
       return im_array[i].im;
-    }
   }
+
   return NULL;
 }
 
 static uim_lisp
 alloc_id(uim_lisp name_)
 {
-  int id = unused_ic_id();
-  char *name = uim_scm_c_str(name_);
-  MInputMethod *im = find_im_by_name(name);
-  if (im) {
+  int id;
+  const char *name;
+  MInputMethod *im;
+
+  id = unused_ic_id();
+  name = uim_scm_refer_c_str(name_);
+
+  im = find_im_by_name(name);
+
+  if (im)
     ic_array[id].mic = minput_create_ic(im, NULL);
-  }
+
   ic_array[id].old_candidates = NULL;
   ic_array[id].new_candidates = NULL;
-  free(name);
+
   return uim_scm_make_int(id);
 }
 
@@ -472,32 +569,37 @@ static uim_lisp
 free_id(uim_lisp id_)
 {
   int id = uim_scm_c_int(id_);
-  if (id < max_input_contexts) {
+
+  if (id < nr_input_contexts) {
     struct ic_ *ic = &ic_array[id];
+
     if (ic->mic) {
       minput_destroy_ic(ic->mic);
       ic->mic = NULL;
     }
   }
+
   return uim_scm_f();
 }
 
 static uim_lisp
 push_symbol_key(uim_lisp id_, uim_lisp key_)
 {
-  int id = uim_scm_c_int(id_);
+  int id;
   MSymbol key;
-  MInputContext *ic = ic_array[id].mic;
-  key = msymbol(uim_scm_c_str(key_));
-  if (key == Mnil) {
-    return uim_scm_t();
-  }
+  MInputContext *ic;
 
-  if(minput_filter(ic, key, NULL)== 1) {
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+  key = msymbol(uim_scm_c_str(key_));
+
+  if (key == Mnil)
     return uim_scm_t();
-  } else {
+
+  if (minput_filter(ic, key, NULL) == 1)
+    return uim_scm_t();
+  else
     return uim_scm_f();
-  }
 }
 
 static uim_lisp
@@ -505,20 +607,20 @@ get_result(uim_lisp id_)
 {
   MText *produced;
   char *commit_string;
-  int consumed;
-  int id = uim_scm_c_int(id_);
-  MInputContext *ic = ic_array[id].mic;
+  int consumed, id;
+  MInputContext *ic;
   uim_lisp  consumed_, commit_string_;
 
-  produced  = mtext();
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
 
+  produced  = mtext();
   consumed  = minput_lookup(ic, NULL, NULL, produced);
 
-  if (consumed == -1) {
+  if (consumed == -1)
     consumed_ = uim_scm_f();
-  } else {
+  else
     consumed_ = uim_scm_t();
-  }
 
   commit_string = convert_mtext2str(produced);
   m17n_object_unref(produced);
@@ -531,23 +633,30 @@ get_result(uim_lisp id_)
 static uim_lisp
 commit(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  MInputContext *ic = ic_array[id].mic;
+  int id;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  /* To avoid a bug of m17n-lib */
+  ic->candidate_to = 0;
 
-/* To avoid a bug of m17n-lib */
-  ic->candidate_to   = 0;
   return uim_scm_f();
 }
 
 static uim_lisp
 candidate_showp(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  MInputContext *ic = ic_array[id].mic;
+  int id;
+  MInputContext *ic;
 
-  if (ic->candidate_show == 1) {
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  if (ic->candidate_show == 1)
     return uim_scm_t();
-  }
+
   return uim_scm_f();
 }
 
@@ -556,7 +665,9 @@ calc_cands_num(int id)
 {
   int result = 0;
   MPlist *group; 
-  MInputContext *ic = ic_array[id].mic;
+  MInputContext *ic;
+
+  ic = ic_array[id].mic;
 
   if (!ic || !ic->candidate_list)
     return 0;
@@ -565,15 +676,14 @@ calc_cands_num(int id)
 
   while (mplist_value(group) != Mnil) {
     if (mplist_key(group) == Mtext) {
-      for (; mplist_key(group) != Mnil; group = mplist_next(group)) {
-        result += mtext_len(mplist_value(group));
-      }
+      for (; mplist_key(group) != Mnil; group = mplist_next(group))
+	result += mtext_len(mplist_value(group));
     } else {
-      for (; mplist_key(group) != Mnil; group = mplist_next(group)) {
-        result += mplist_length(mplist_value(group));
-      }
+      for (; mplist_key(group) != Mnil; group = mplist_next(group))
+	result += mplist_length(mplist_value(group));
     }
   }
+
   return result;
 }
 
@@ -581,10 +691,10 @@ static void
 old_cands_free(char **old_cands)
 {
   int i = 0;
+
   if (old_cands) {
-    for (i=0; old_cands[i]; i++) {
+    for (i = 0; old_cands[i]; i++)
       free(old_cands[i]);
-    }
     free(old_cands);
   }
 }
@@ -592,16 +702,15 @@ old_cands_free(char **old_cands)
 static uim_lisp
 fill_new_candidates(uim_lisp id_)
 {
-  MText *produced = NULL; /* Quiet gcc */
-  MPlist *group;
-  MPlist *elm;
-  int i;
-  char *buf = NULL; /* Quiet gcc */
-  int id = uim_scm_c_int(id_);
-  MInputContext *ic = ic_array[id].mic;
-  int cands_num = calc_cands_num(id);
+  MText *produced;
+  MPlist *group, *elm;
+  int i, id, cands_num;
   char **new_cands;
-  uim_lisp buf_;
+  MInputContext *ic;
+
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+  cands_num = calc_cands_num(id);
 
   if (!ic || !ic->candidate_list)
     return uim_scm_f();
@@ -614,20 +723,20 @@ fill_new_candidates(uim_lisp id_)
   new_cands = malloc (cands_num * sizeof(char *) + 2);
 
   if (mplist_key(group) == Mtext) {
-
-    for (i=0; mplist_key(group) != Mnil; group = mplist_next(group)) {
+    for (i = 0; mplist_key(group) != Mnil; group = mplist_next(group)) {
       int j;
-      for (j=0; j < mtext_len(mplist_value(group)); j++, i++) {
-          produced = mtext();
-          mtext_cat_char(produced, mtext_ref_char(mplist_value(group), j));
-          new_cands[i] = convert_mtext2str(produced);
-          m17n_object_unref(produced);
+      for (j = 0; j < mtext_len(mplist_value(group)); j++, i++) {
+	  produced = mtext();
+	  mtext_cat_char(produced, mtext_ref_char(mplist_value(group), j));
+	  new_cands[i] = convert_mtext2str(produced);
+	  m17n_object_unref(produced);
       }
     }
   } else {
-    for (i=0; mplist_key(group) != Mnil; group = mplist_next(group)) {
+    for (i = 0; mplist_key(group) != Mnil; group = mplist_next(group)) {
 
-      for (elm = mplist_value(group); mplist_key(elm) != Mnil; elm = mplist_next(elm),i++) {
+      for (elm = mplist_value(group); mplist_key(elm) != Mnil;
+	   elm = mplist_next(elm),i++) {
 	produced = mplist_value(elm);
 	new_cands[i] = convert_mtext2str(produced);
       }
@@ -639,28 +748,21 @@ fill_new_candidates(uim_lisp id_)
   ic_array[id].nr_candidates = i;
   
   return uim_scm_t();
-
-  if (!buf) {
-    return uim_scm_make_str("");
-  } else {
-    buf_ = uim_scm_make_str(buf);
-    free(buf);
-    return buf_;
-  }
 }
 
 static uim_bool
-same_candidatesp(const char **old, const char **new)
+same_candidatesp(char **old, char **new)
 {
   int i;
+
   if (!old)
     return UIM_FALSE;
 
-  for (i=0; old[i] && new[i]; i++) {
-    if (strcmp(old[i], new[i]) != 0) {
+  for (i = 0; old[i] && new[i]; i++) {
+    if (strcmp(old[i], new[i]) != 0)
       return UIM_FALSE;
-    }
   }
+
   return UIM_TRUE;
 }
 
@@ -668,12 +770,11 @@ static uim_lisp
 candidates_changedp(uim_lisp id_)
 {
   int id = uim_scm_c_int(id_);
-  char **old_cands = ic_array[id].old_candidates;
-  char **new_cands = ic_array[id].new_candidates;
 
-  if (!same_candidatesp(old_cands, new_cands)) {
+  if (!same_candidatesp(ic_array[id].old_candidates,
+			ic_array[id].new_candidates))
     return uim_scm_t();
-  }
+
   return uim_scm_f();
 }
 
@@ -681,59 +782,62 @@ static uim_lisp
 get_nr_candidates(uim_lisp id_)
 {
   int id = uim_scm_c_int(id_);
+
   return uim_scm_make_int(calc_cands_num(id));
 }
 
 static uim_lisp
 get_nth_candidate(uim_lisp id_, uim_lisp nth_)
 {
-  int id = uim_scm_c_int(id_);
-  int nth = uim_scm_c_int(nth_);
-  int nr = ic_array[id].nr_candidates;
+  int id, nth, nr;
   
-  if (nr >= nth) {
+  id = uim_scm_c_int(id_);
+  nth = uim_scm_c_int(nth_);
+  nr = ic_array[id].nr_candidates;
+
+  if (nr >= nth)
     return uim_scm_make_str(ic_array[id].new_candidates[nth]);
-  } else {
+  else
     return uim_scm_make_str("");
-  }
 }
 
 static uim_lisp
 get_candidate_index(uim_lisp id_)
 {
-  int id = uim_scm_c_int(id_);
-  MInputContext *ic = ic_array[id].mic;
-  return uim_scm_make_int(ic->candidate_index);
-}
+  int id;
+  MInputContext *ic;
 
-/* Utility function */
-static char *
-m17nlib_utf8_find_next_char(const char *p)
-{
-  if (*p) {
-    for (++p; (*p & 0xc0) == 0x80; ++p)
-      ;
-  }
-  return (char *)p;
+  id = uim_scm_c_int(id_);
+  ic = ic_array[id].mic;
+
+  return uim_scm_make_int(ic->candidate_index);
 }
 
 void
 uim_plugin_instance_init(void)
 {
   uim_scm_init_subr_0("m17nlib-lib-init", init_m17nlib);
-  uim_scm_init_subr_0("m17nlib-lib-nr-input-methods", get_nr_input_methods);
-  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-lang", get_input_method_lang);
-  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-name", get_input_method_name);
+  uim_scm_init_subr_0("m17nlib-lib-nr-input-methods",
+		      get_nr_input_methods);
+  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-lang",
+		      get_input_method_lang);
+  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-name",
+		      get_input_method_name);
+  uim_scm_init_subr_1("m17nlib-lib-nth-input-method-short-desc",
+		      get_input_method_short_desc);
   uim_scm_init_subr_1("m17nlib-lib-alloc-context", alloc_id);
   uim_scm_init_subr_1("m17nlib-lib-free-context", free_id);
   uim_scm_init_subr_2("m17nlib-lib-push-symbol-key", push_symbol_key);
   uim_scm_init_subr_1("m17nlib-lib-compose-mode?", compose_modep);
   uim_scm_init_subr_1("m17nlib-lib-preedit-changed?", preedit_changedp);
-  uim_scm_init_subr_1("m17nlib-lib-get-left-of-cursor",     get_left_of_cursor);
-  uim_scm_init_subr_1("m17nlib-lib-get-right-of-cursor",    get_right_of_cursor);
-  uim_scm_init_subr_1("m17nlib-lib-get-left-of-candidate",  get_left_of_candidate);
-  uim_scm_init_subr_1("m17nlib-lib-get-selected-candidate", get_selected_candidate);
-  uim_scm_init_subr_1("m17nlib-lib-get-right-of-candidate", get_right_of_candidate);
+  uim_scm_init_subr_1("m17nlib-lib-get-left-of-cursor", get_left_of_cursor);
+  uim_scm_init_subr_1("m17nlib-lib-get-right-of-cursor", get_right_of_cursor);
+  uim_scm_init_subr_1("m17nlib-lib-get-left-of-candidate",
+		      get_left_of_candidate);
+  uim_scm_init_subr_1("m17nlib-lib-get-selected-candidate",
+		      get_selected_candidate);
+  uim_scm_init_subr_1("m17nlib-lib-get-right-of-candidate",
+		      get_right_of_candidate);
   uim_scm_init_subr_1("m17nlib-lib-get-result", get_result);
   uim_scm_init_subr_1("m17nlib-lib-commit", commit);
   uim_scm_init_subr_1("m17nlib-lib-candidate-show?", candidate_showp);
