@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -30,7 +30,7 @@
   SUCH DAMAGE.
 
 */
-#include "config.h"
+#include <config.h>
 
 #define Uses_SCIM_CONFIG_PATH
 #define Uses_SCIM_IMENGINE_MODULE
