@@ -1,6 +1,5 @@
-/*
-
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+/* 
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -36,7 +35,7 @@
  *
  * Many many things are to be implemented!
  */
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/mman.h>
@@ -53,6 +52,12 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#endif
 
 #include "uim-scm.h"
 #include "uim-helper.h"
@@ -692,7 +697,7 @@ search_line_from_server(struct dic_info *di, const char *s, char okuri_head)
 {
   char r;
   struct skk_line *sl;
-  int n = 0, ret;
+  int n = 0, ret, len;
   char buf[SKK_SERV_BUFSIZ];
   char *line;
   char *idx = alloca(strlen(s) + 2);
@@ -729,15 +734,18 @@ search_line_from_server(struct dic_info *di, const char *s, char okuri_head)
       }
 
       if (r == '\n') {
-	line = realloc(line, strlen(line) + n + 1);
-	strncat(line, buf, n);
+	len = strlen(line) + n;
+	line = realloc(line, len + 1);
+	strlcat(line, buf, len + 1);
 	break;
       }
 
       buf[n] = r;
-      if (n == SKK_SERV_BUFSIZ - 1) {
-	line = realloc(line, strlen(line) + n + 2);
-	strncat(line, buf, n + 1);
+      buf[n + 1] = '\0';
+      if (n == SKK_SERV_BUFSIZ - 2) {
+	len = strlen(line) + n + 1;
+	line = realloc(line, len + 1);
+	strlcat(line, buf, len + 1);
 	n = 0;
       } else {
 	n++;
@@ -777,6 +785,7 @@ search_line_from_file(struct dic_info *di, const char *s, char okuri_head)
   p = find_line(di, n);
   len = calc_line_len(p);
   line = malloc(len + 1);
+  /* strncat is used intentionally because *p is too long string */
   line[0] = '\0';
   strncat(line, p, len);
   sl = compose_line(di, s, okuri_head, line);
@@ -860,7 +869,7 @@ find_cand_array_lisp(uim_lisp head_, uim_lisp okuri_head_, uim_lisp okuri_,
 
   hs = uim_scm_refer_c_str(head_);
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     rs = replace_numeric(hs);
 
   if (okuri_ != uim_scm_null_list())
@@ -998,8 +1007,7 @@ get_purged_words(const char *str)
 	  words = realloc(words, sizeof(char *) * nr);
 	else
 	  words = malloc(sizeof(char *));
-	strncpy(orig, word, len);
-	orig[len] = '\0';
+	strlcpy(orig, word, len + 1);
 
 	expanded_word = expand_str(orig);
 	if (expanded_word)
@@ -1094,7 +1102,7 @@ skk_get_entry(uim_lisp head_, uim_lisp okuri_head_,
   if (ca && ca->nr_cands > 0 && !is_purged_only(ca))
       return uim_scm_t();
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     return skk_get_entry(head_, okuri_head_, okuri_, uim_scm_f());
 
   return uim_scm_f();
@@ -1130,8 +1138,7 @@ skk_store_replaced_numeric_str(uim_lisp head_)
 	  numstr = malloc(numlen + 1);
 	else
 	  numstr = realloc(numstr, numlen + 1);
-	strncpy(numstr, &str[start], numlen);
-	numstr[numlen] = '\0';
+	strlcpy(numstr, &str[start], numlen + 1);
 	lst = uim_scm_cons(uim_scm_make_str(numstr), lst);
       }
       prev_is_num = 0;
@@ -1147,8 +1154,7 @@ skk_store_replaced_numeric_str(uim_lisp head_)
       numstr = malloc(numlen + 1);
     else
       numstr = realloc(numstr, numlen + 1);
-    strncpy(numstr, &str[start], numlen);
-    numstr[numlen] = '\0';
+    strlcpy(numstr, &str[start], numlen + 1);
     lst = uim_scm_cons(uim_scm_make_str(numstr), lst);
   }
   free(numstr);
@@ -1631,7 +1637,7 @@ skk_get_nth_candidate(uim_lisp nth_, uim_lisp head_, uim_lisp okuri_head_, uim_l
   uim_lisp numlst_ = uim_scm_null_list();
   int ignoring_indices[IGNORING_WORD_MAX + 1];
   
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     numlst_ = skk_store_replaced_numeric_str(head_);
 
   n = uim_scm_c_int(nth_);
@@ -1720,7 +1726,7 @@ skk_get_nr_candidates(uim_lisp head_, uim_lisp okuri_head_, uim_lisp okuri_, uim
   uim_lisp numlst_ = uim_scm_null_list();
   int ignoring_indices[IGNORING_WORD_MAX + 1];
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     numlst_ = skk_store_replaced_numeric_str(head_);
 
   if (!uim_scm_nullp(numlst_))
@@ -1789,7 +1795,7 @@ make_comp_array_from_cache(struct dic_info *di, const char *s, uim_lisp use_look
     }
   }
 
-  if NFALSEP(use_look_)
+  if UIM_SCM_NFALSEP(use_look_)
     look_get_comp(ca, s);
 
   if (ca->nr_comps == 0) {
@@ -1831,7 +1837,7 @@ find_comp_array_lisp(uim_lisp head_, uim_lisp numeric_conv_, uim_lisp use_look_)
   
   hs = uim_scm_refer_c_str(head_);
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     rs = replace_numeric(hs);
 
   if (!rs)
@@ -1853,7 +1859,7 @@ skk_get_completion(uim_lisp head_, uim_lisp numeric_conv_, uim_lisp use_look_)
     return uim_scm_t();
   }
 
-  if (NFALSEP(numeric_conv_) && has_numeric_in_head(head_))
+  if (UIM_SCM_NFALSEP(numeric_conv_) && has_numeric_in_head(head_))
     return skk_get_completion(head_, uim_scm_f(), use_look_);
 
   return uim_scm_f();
@@ -1868,7 +1874,7 @@ skk_get_nth_completion(uim_lisp nth_, uim_lisp head_,
   char *str;
   uim_lisp numlst_ = uim_scm_null_list();
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     numlst_ = skk_store_replaced_numeric_str(head_);
 
   if (!uim_scm_nullp(numlst_))
@@ -1909,7 +1915,7 @@ skk_get_nr_completions(uim_lisp head_, uim_lisp numeric_conv_, uim_lisp use_look
   if (ca)
     n = ca->nr_comps;
 
-  if (NFALSEP(numeric_conv_) && has_numeric_in_head(head_))
+  if (UIM_SCM_NFALSEP(numeric_conv_) && has_numeric_in_head(head_))
     return uim_scm_make_int(n +
 		    uim_scm_c_int(skk_get_nr_completions(head_, uim_scm_f(), use_look_))); 
 
@@ -1926,7 +1932,7 @@ skk_clear_completions(uim_lisp head_, uim_lisp numeric_conv_)
 
   hs = uim_scm_refer_c_str(head_);
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     rs = replace_numeric(hs);
 
   if (!rs)
@@ -1966,7 +1972,7 @@ skk_clear_completions(uim_lisp head_, uim_lisp numeric_conv_)
     }
   }
 
-  if (NFALSEP(numeric_conv_) && has_numeric_in_head(head_))
+  if (UIM_SCM_NFALSEP(numeric_conv_) && has_numeric_in_head(head_))
     skk_clear_completions(head_, uim_scm_f());
 
   return uim_scm_t();
@@ -2017,7 +2023,7 @@ skk_get_dcomp_word(uim_lisp head_, uim_lisp numeric_conv_, uim_lisp use_look_)
   numlst_ = uim_scm_null_list();
   hs = uim_scm_refer_c_str(head_);
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     numlst_ = skk_store_replaced_numeric_str(head_);
 
   if (!uim_scm_nullp(numlst_)) {
@@ -2035,9 +2041,9 @@ skk_get_dcomp_word(uim_lisp head_, uim_lisp numeric_conv_, uim_lisp use_look_)
 			sl->state & SKK_LINE_USE_FOR_COMPLETION)
 	  return uim_scm_make_str(sl->head);
       }
-      if NFALSEP(use_look_) {
+      if UIM_SCM_NFALSEP(use_look_) {
 	look_ = look_get_top_word(hs);
-	if NFALSEP(look_)
+	if UIM_SCM_NFALSEP(look_)
 	  return look_;
       }
     } else {
@@ -2049,10 +2055,10 @@ skk_get_dcomp_word(uim_lisp head_, uim_lisp numeric_conv_, uim_lisp use_look_)
 	  return restore_numeric(sl->head, numlst_);
 	}
       }
-      if NFALSEP(use_look_) {
+      if UIM_SCM_NFALSEP(use_look_) {
 	look_ = look_get_top_word(rs);
 	free(rs);
-	if NFALSEP(look_)
+	if UIM_SCM_NFALSEP(look_)
 	  return look_;
       } else {
 	free(rs);
@@ -2347,7 +2353,7 @@ skk_commit_candidate(uim_lisp head_, uim_lisp okuri_head_,
   uim_lisp numlst_ = uim_scm_null_list();
   int ignoring_indices[IGNORING_WORD_MAX + 1];
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     numlst_ = skk_store_replaced_numeric_str(head_);
 
   nth = uim_scm_c_int(nth_);
@@ -2494,7 +2500,7 @@ skk_purge_candidate(uim_lisp head_, uim_lisp okuri_head_,
   uim_lisp numlst_ = uim_scm_null_list();
   int ignoring_indices[IGNORING_WORD_MAX + 1];
 
-  if NFALSEP(numeric_conv_)
+  if UIM_SCM_NFALSEP(numeric_conv_)
     numlst_ = skk_store_replaced_numeric_str(head_);
 
   if (!uim_scm_nullp(numlst_))
@@ -3257,8 +3263,7 @@ eval_candidate_with_concat(const char *cand)
   /* get quoted str  */
   len = (q - p + 1) - strlen("(concat \"\")");
   str = malloc(len + 1);
-  strncpy(str, p + strlen("(concat \""), len);
-  str[len] = '\0';
+  strlcpy(str, p + strlen("(concat \""), len + 1);
 
   expanded_str = expand_str(str);
   if (!expanded_str) {
@@ -3272,8 +3277,7 @@ eval_candidate_with_concat(const char *cand)
     str = realloc(str, len + 1);
 
   if (p != cand) {
-    strncpy(str, cand, p - cand);
-    str[p - cand] = '\0';
+    strlcpy(str, cand, p - cand + 1);
     strcat(str, expanded_str);
   } else {
     strcpy(str, expanded_str);
