@@ -2,7 +2,7 @@
 
   uim-ipc.c: Utility functions for inter process communication.
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -193,7 +193,7 @@ uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
       }
       *ap = NULL;
     }
-    if (is_setugid() != 0) {
+    if (uim_issetugid()) {
       int cmd_len = strlen(command) + 30;
       char *fullpath_command = malloc(cmd_len);
       char *cmd_name = strrchr(command, '/');
@@ -206,15 +206,15 @@ uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
       
       snprintf(fullpath_command, cmd_len, "/usr/local/bin/%s", cmd_name);
 
-      result = execvp(fullpath_command, argv);
+      result = execv(fullpath_command, argv);
 
       if (result == -1) {
  	snprintf(fullpath_command, cmd_len, "/usr/bin/%s", cmd_name);
-	result = execvp(fullpath_command, argv);
+	result = execv(fullpath_command, argv);
       }
       if (result == -1) {
  	snprintf(fullpath_command, cmd_len, UIM_LIBEXECDIR "/%s", cmd_name);
-	result = execvp(fullpath_command, argv);
+	result = execv(fullpath_command, argv);
       }
       free(fullpath_command);
     } else {
