@@ -1,7 +1,7 @@
 /*
   uim-module-manager.c: source file for uim-module-manager.
 
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -32,14 +32,14 @@
 
 */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "config.h"
-
 #include "uim.h"
 #include "uim-scm.h"
 #include "uim-internal.h"
@@ -74,8 +74,8 @@ concat(const char *a, const char *b)
     return NULL;
   len = strlen(a) + strlen(b) + 1;
   dest = malloc(len);
-  dest = strcpy(dest, a);
-  strcat(dest, b);
+  strlcpy(dest, a, len);
+  strlcat(dest, b, len);
   return dest;
 }
 
@@ -248,5 +248,5 @@ main(int argc, char *argv[]) {
 
   uim_quit();
 
-  return 0;
+  return EXIT_SUCCESS;
 }
