@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,7 +31,8 @@
 
 */
 
-#include "config.h"
+#include <config.h>
+
 #include <pwd.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -58,6 +59,7 @@ char *uim_last_client_encoding;
 static uim_context context_array[CONTEXT_ARRAY_SIZE];
 struct uim_im *uim_im_array;
 int uim_nr_im;
+
 static int uim_initialized;
 
 void
@@ -140,7 +142,6 @@ uim_create_context(void *ptr,
   uc->mode = 0;
   /**/
   uc->propstr = NULL;
-  uc->proplabelstr = NULL;
   /**/
   uc->preedit_clear_cb = NULL;
   uc->preedit_pushback_cb = NULL;
@@ -151,7 +152,6 @@ uim_create_context(void *ptr,
   uc->mode_update_cb = NULL;
   /**/
   uc->prop_list_update_cb  = NULL;
-  uc->prop_label_update_cb = NULL;
   /**/
   uc->candidate_selector_activate_cb = NULL;
   uc->candidate_selector_select_cb = NULL;
@@ -163,6 +163,9 @@ uim_create_context(void *ptr,
   /**/
   uc->configuration_changed_cb = NULL;
   /**/
+  uc->switch_app_global_im_cb = NULL;
+  uc->switch_system_global_im_cb = NULL;
+  /**/
   uc->nr_candidates = 0;
   uc->candidate_index = 0;
   /**/
@@ -212,6 +215,15 @@ uim_set_configuration_changed_cb(uim_context uc,
   uc->configuration_changed_cb = changed_cb;
 }
 
+void
+uim_set_im_switch_request_cb(uim_context uc,
+			     void (*sw_app_im_cb)(void *ptr, const char *name),
+			     void (*sw_system_im_cb)(void *ptr, const char *name))
+{
+  uc->switch_app_global_im_cb = sw_app_im_cb;
+  uc->switch_system_global_im_cb = sw_system_im_cb;
+}
+
 void
 uim_switch_im(uim_context uc, const char *engine)
 {
@@ -268,7 +280,6 @@ uim_release_context(uim_context uc)
     uc->modes[i] = NULL;
   }
   free(uc->propstr);
-  free(uc->proplabelstr);
   free(uc->modes);
   free(uc->short_desc);
   free(uc->encoding);
@@ -306,7 +317,6 @@ uim_set_mode_list_update_cb(uim_context uc,
   uc->mode_list_update_cb = update_cb;
 }
 
-
 void
 uim_set_prop_list_update_cb(uim_context uc,
 			    void (*update_cb)(void *ptr, const char *str))
@@ -314,15 +324,13 @@ uim_set_prop_list_update_cb(uim_context uc,
   uc->prop_list_update_cb = update_cb;
 }
 
-
+/* Obsolete */
 void
 uim_set_prop_label_update_cb(uim_context uc,
 			     void (*update_cb)(void *ptr, const char *str))
 {
-  uc->prop_label_update_cb = update_cb;
 }
 
-
 void
 uim_prop_activate(uim_context uc, const char *str)
 {
@@ -390,15 +398,14 @@ uim_set_mode_cb(uim_context uc, void (*update_cb)(void *ptr,
 void
 uim_prop_list_update(uim_context uc)
 {
-  if (uc && uc->propstr)
+  if (uc && uc->propstr && uc->prop_list_update_cb)
     uc->prop_list_update_cb(uc->ptr, uc->propstr);
 }
 
+/* Obsolete */
 void
 uim_prop_label_update(uim_context uc)
 {
-  if (uc && uc->proplabelstr)
-    uc->prop_label_update_cb(uc->ptr, uc->proplabelstr);
 }
 
 int
@@ -631,7 +638,7 @@ uim_init_scm(void)
   char *scm_files = NULL;
   char *env = NULL;
 
-  /*  if (is_setugid() == 0) {*/
+  /*  if (!uim_issetugid()) {*/
     env = getenv("LIBUIM_VERBOSE");
     /*  }*/
   uim_scm_init(env);  /* init Scheme interpreter */
@@ -642,10 +649,13 @@ uim_init_scm(void)
   uim_init_intl_subrs();
   uim_init_util_subrs();
   uim_init_plugin();
+#ifdef ENABLE_ANTHY_STATIC
+  uim_anthy_plugin_instance_init();
+#endif
   uim_init_im_subrs();
   uim_init_key_subrs();
   
-  if (is_setugid() == 0) {
+  if (!uim_issetugid()) {
     scm_files = getenv("LIBUIM_SCM_FILES");
   }
   uim_scm_set_lib_path((scm_files) ? scm_files : SCM_FILES);
@@ -708,6 +718,9 @@ uim_quit(void)
   }
   /**/
   uim_quit_plugin();
+#ifdef ENABLE_ANTHY_STATIC
+  uim_anthy_plugin_instance_quit();
+#endif
   uim_scm_quit();
   free(uim_last_client_encoding);
   uim_last_client_encoding = NULL;
