@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -17,10 +17,10 @@
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
 
-  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -35,17 +35,41 @@
 #ifndef _uim_h_included_
 #define _uim_h_included_
 
+#include <stdio.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#include <stdio.h>
+#define UIM_VERSION_MAJOR      (1)
+#define UIM_VERSION_MINOR      (2)
+#define UIM_VERSION_PATCHLEVEL (1)
+#define UIM_VERSION_REQUIRE(major, minor, patchlevel)			     \
+  ((major) < UIM_VERSION_MAJOR						     \
+   || ((major) == UIM_VERSION_MAJOR && (minor) < UIM_VERSION_MINOR)	     \
+   || ((major) == UIM_VERSION_MAJOR && (minor) == UIM_VERSION_MINOR	     \
+       && (patchlevel) <= UIM_VERSION_PATCHLEVEL))
+
+/*
+ * A boolean type for uim to explicitly indicate intention about values
+ *
+ *                           *** IMPORTANT ***
+ *
+ * Do not test a value with (val == UIM_TRUE). The UIM_TRUE is only A TYPICAL
+ * VALUE FOR TRUE. Use (val) or (val != UIM_FALSE) instead.
+ *
+ */
+typedef int uim_bool;
+
+#define UIM_FALSE 0
+#define UIM_TRUE 1
+
 
 typedef struct uim_context_ *uim_context;
 
 typedef struct uim_candidate_ *uim_candidate;
 
-/* This will be filled later, so ABI is unstable */
+/* FIXME: We need reorganize of key event handling...*/
 enum UKey {
   UKey_0 = 48,
   UKey_1 = 49,
@@ -70,11 +94,24 @@ enum UKey {
   UKey_Next , /* page down */
   UKey_Home,
   UKey_End,
-  UKey_Zenkaku_Hankaku, /* zenkaku/hankaku toggle */
   UKey_Multi_key, /* multi-key character compose */
   UKey_Mode_switch, /* charcter set switch */
-  UKey_Henkan_Mode, /* start/stop conversion */
+  UKey_Kanji, /* kanji, kanji convert */
   UKey_Muhenkan, /* cancel conversion */
+  UKey_Henkan_Mode, /* start/stop conversion */
+  UKey_Romaji,
+  UKey_Hiragana,
+  UKey_Katakana,
+  UKey_Hiragana_Katakana, /* hiragana/katakana toggle */
+  UKey_Zenkaku,
+  UKey_Hankaku,
+  UKey_Zenkaku_Hankaku, /* zenkaku/hankaku toggle */
+  UKey_Touroku,
+  UKey_Massyo,
+  UKey_Kana_Lock,
+  UKey_Kana_Shift,
+  UKey_Eisu_Shift, /* alphanumeric shift */
+  UKey_Eisu_toggle, /* alphanumeric toggle */
   UKey_F1,
   UKey_F2,
   UKey_F3,
@@ -149,6 +186,7 @@ enum UKey {
   UKey_Meta_key,
   UKey_Super_key,
   UKey_Hyper_key,
+  UKey_Insert,
   UKey_Other = 1000
 };
   
@@ -161,17 +199,6 @@ enum UKeyModifier {
   UMod_Pseudo1 = 32,
   UMod_Super = 64,
   UMod_Hyper = 128
-
-#ifndef UIM_NO_BACKWARD_COMPATIBLE_API
-  /* will be deprecated */
-  ,
-  UKey_Shift = UMod_Shift,
-  UKey_Control = UMod_Control,
-  UKey_Alt = UMod_Alt,
-  UKey_Meta = UMod_Meta,
-  UKey_Pseudo0 = UMod_Pseudo0,
-  UKey_Pseudo1 = UMod_Pseudo1
-#endif
 };
   
 enum UPreeditAttr {
@@ -180,16 +207,6 @@ enum UPreeditAttr {
   UPreeditAttr_Reverse = 2,
   UPreeditAttr_Cursor = 4,
   UPreeditAttr_Separator = 8
-
-#ifndef UIM_NO_BACKWARD_COMPATIBLE_API
-  /* will be deprecated */
-  ,
-  UPeAttr_None = UPreeditAttr_None,
-  UPeAttr_UnderLine = UPreeditAttr_UnderLine,
-  UPeAttr_Reverse = UPreeditAttr_Reverse,
-  UPeAttr_Cursor = UPreeditAttr_Cursor,
-  UPeAttr_Separator = UPreeditAttr_Separator
-#endif
 };
 
 /* abstracting platform-dependent character code conversion method */
@@ -223,7 +240,7 @@ uim_quit(void);
  * Currently selected input method is used if you specify both as NULL.
  *
  * @param ptr cookie value which is passed as an argument of uim's callback functions.
- * @param enc name of client encoding, say "UTF-8"
+ * @param enc iconv-acceptable name of client encoding. Say "UTF-8" to use most of input methods.
  * @param lang name language you want to input
  * @param engine name of conversion engine you want to use
  * @param conv character code converter. Say "uim_iconv" or place your own platform-specific, preferable implementation. See struct uim_code_converter.
@@ -433,9 +450,9 @@ void  uim_set_candidate_index(uim_context uc, int index);
 /**
  * Get the string of candidate.
  *
- * @param cand the data you got by uim_get_candidate
+ * @warning You must not free the result. All datas are freed by calling uim_candidate_free.
  *
- * @warning You must not free the result.
+ * @param cand the data you got by calling uim_get_candidate
  *
  * @see uim_get_candidate
  *
@@ -445,16 +462,29 @@ const char *uim_candidate_get_cand_str(uim_candidate cand);
 /**
  * Get the string of candidate's heading label.
  *
- * @param cand the data you got by uim_get_candidate
+ * @warning You must not free the result. All datas are freed by calling uim_candidate_free.
  *
- * @warning You must not free the result.
+ * @param cand the data you got by uim_get_candidate
  *
  * @see uim_get_candidate
  *
  * @return string of candidate data's heading label
  */
 const char *uim_candidate_get_heading_label(uim_candidate cand);
-  
+
+/**
+ * Get the string of candidate's annotation.
+ *
+ * @warning You must not free the result. All datas are freed by calling uim_candidate_free.
+ * @warning If no data is available, return string is "" (empty string).
+ *
+ * @param cand the data you got by uim_get_candidate
+ *
+ * @see uim_get_candidate
+ *
+ * @return string of candidate's annotation str
+ */
+const char *uim_candidate_get_annotation_str(uim_candidate cand);
 
 /*property*/
 /**
@@ -477,20 +507,15 @@ void
 uim_prop_list_update(uim_context uc);
 
 /**
- * Set callback function to be called when property list is updated.
- *
- * @param uc input context
- * @param update_cb called when property list is updated.
- *        1st argument "ptr" corresponds to the 1st argument of uim_create_context.
- *        2nd argument is the message to be sent to the helper server with "prop_label_update" command and charset info.
+ * Obsolete. Only existing for Backward compatibility and should not
+ * be called.
  */
 void
 uim_set_prop_label_update_cb(uim_context uc,
 			     void (*update_cb)(void *ptr, const char *str));
 /**
- * Update property label
- *
- * @param uc input context
+ * Obsolete. Only existing for Backward compatibility and should not
+ * be called.
  */
 void
 uim_prop_label_update(uim_context uc);
@@ -498,6 +523,8 @@ void
 uim_prop_activate(uim_context uc, const char *str);
 void
 uim_prop_update_custom(uim_context uc, const char *custom, const char *val);
+uim_bool
+uim_prop_reload_configs(void);
 
 /* mode is obsoleted by property, so please use property API instead of mode API */
 int
@@ -540,15 +567,59 @@ void
 uim_set_surrounding_text(uim_context uc, const char *text,
 			 int cursor_pos, int len);
 
+/*
+ * Set callback function to be called when configuration of input
+ * context is changed.
+ *
+ * @param uc input context
+ * @param changed_cb called when configuration of the input context is changed.
+ *        1st argument "ptr" corresponds to the 1st argument of uim_create_context.
+ */
+void
+uim_set_configuration_changed_cb(uim_context uc,
+				 void (*changed_cb)(void *ptr));
+
+/*
+ * Set callback functions called when IM-switching of specific set of context
+ * is requested.
+ *
+ * When the functions are called back, bridges should re-initialize the
+ * specified input contexts with the IM specified by 2nd argument
+ * 'name'. Since the re-initialization method of specified contexts vary for
+ * each IM environment, it is delegated to bridges via the callback. For
+ * example, ordinary desktop system should send the helper message
+ * im_change_whole_desktop in response to @a sw_system_im_cb. But in embedded
+ * systems such as Qtopia, nothing to do with @a sw_system_im_cb since only
+ * one input context is running on the window system and so system-global.
+ *
+ * @param uc input context
+
+ * @param sw_app_im_cb called when re-initialization of all contexts within
+ *        the application that @a uc belongs to, with specified IM is
+ *        requested. 1st argument "ptr" corresponds to the 1st argument of
+ *        uim_create_context, and 2nd "name" is requested idname of IM. The
+ *        originating context is supposed to already be switched, and must not
+ *        switched by the callback.
+ * @param sw_system_im_cb called when re-initialization of all contexts
+ *        running on the system that @a uc is running on, with specified IM is
+ *        requested. The originating context is supposed to already be
+ *        switched.
+ */
+void
+uim_set_im_switch_request_cb(uim_context uc,
+			     void (*sw_app_im_cb)(void *ptr, const char *name),
+			     void (*sw_system_im_cb)(void *ptr, const char *name));
 
 /* Utility functions */
 int
-uim_ipc_open_command(int old_pid, FILE **read_handler, FILE **write_handler, char *command);
+uim_ipc_open_command(int old_pid, FILE **read_handler, FILE **write_handler, const char *command);
+int
+uim_ipc_open_command_with_option(int old_pid, FILE **read_handler, FILE **write_handler, const char *command, const char *option);
 char *
 uim_ipc_send_command(int *pid, FILE **read_handler,
-		     FILE **write_handler, char *command, char *str);
+		     FILE **write_handler, const char *command, const char *str);
 
-/* will be deprecated. use uim-scm.h (still preliminary) instead */
+/* will be deprecated. use custom API (uim-custom.h) instead */
 char *uim_symbol_value_str(const char *symbol_str);
 
 /* an uim_code_converter implementation using iconv */
