@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -30,10 +30,8 @@
   SUCH DAMAGE.
 */
 
-#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
-
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <stdlib.h>
@@ -66,7 +64,7 @@ static FILE *candwin_r, *candwin_w;
 static int candwin_pid;
 static Canddisp *disp;
 static const char *command;
-static bool candwin_inited = false;
+static bool candwin_initted = false;
 
 static void candwin_read_cb(int fd, int ev);
 
@@ -104,7 +102,7 @@ Canddisp *canddisp_singleton()
     if (!command)
 	command = candwin_command();
 
-    if (!candwin_inited && command) {
+    if (!candwin_initted && command) {
 	candwin_pid = uim_ipc_open_command(candwin_pid, &candwin_r, &candwin_w, command);
 	if (disp)
 	    delete disp;
@@ -118,7 +116,7 @@ Canddisp *canddisp_singleton()
 		    add_fd_watch(fd, READ_OK, candwin_read_cb);
 	    }
 	}
-	candwin_inited = true;
+	candwin_initted = true;
     }
     return disp;
 }
@@ -139,7 +137,7 @@ void Canddisp::activate(std::vector<const char *> candidates, int display_limit)
 
     fprintf(candwin_w, "activate\ncharset=UTF-8\ndisplay_limit=%d\n",
 		    display_limit);
-    for (i = candidates.begin(); i != candidates.end(); i++)
+    for (i = candidates.begin(); i != candidates.end(); ++i)
 	fprintf(candwin_w, "%s\n", *i);
     fprintf(candwin_w, "\n");
     fflush(candwin_w);
@@ -201,7 +199,7 @@ void Canddisp::show_caret_state(const char *str, int timeout)
 	return;
     fprintf(candwin_w, "show_caret_state\n");
     fprintf(candwin_w, "%d\n", timeout);
-    fprintf(candwin_w, "%s", str);
+    fprintf(candwin_w, "%s\n", str);
     fprintf(candwin_w, "\n");
     fflush(candwin_w);
     check_connection();
@@ -231,7 +229,7 @@ void Canddisp::check_connection()
 	terminate_canddisp_connection();
 }
 
-static void candwin_read_cb(int fd, int ev)
+static void candwin_read_cb(int fd, int /* ev */)
 {
     char buf[1024];
     int n;
@@ -289,6 +287,6 @@ void terminate_canddisp_connection()
     }
 
     candwin_w = candwin_r = NULL;
-    candwin_inited = false;
+    candwin_initted = false;
     return;
 }
