@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -32,10 +32,8 @@
 
 // Locale dependent routines
 
-#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
-
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <stdio.h>
@@ -169,14 +167,14 @@ ustring_to_utf8_str(uString *s)
     int l = 0, nbyte;
     unsigned char utf8[6];
     // count the length
-    for (i = s->begin(); i != s->end(); i++) {
+    for (i = s->begin(); i != s->end(); ++i) {
 	nbyte = utf8_wctomb(utf8, *i);
 	l += nbyte;
     }
     char *c = (char *)malloc(l + 1);
     c[l] = 0;
     l = 0;
-    for (i = s->begin(); i != s->end(); i++) {
+    for (i = s->begin(); i != s->end(); ++i) {
 	nbyte = utf8_wctomb(utf8, *i);
 	int j;
 	for (j = 0; j < nbyte; j++) {
@@ -284,8 +282,7 @@ char *UTF8_Locale::utf8_to_native_str(char *utf8) {
 	return NULL;
     }
 
-    strncpy(convstr, outbuf, outbufsize - outbytesleft);
-    convstr[outbufsize - outbytesleft] = '\0';
+    strlcpy(convstr, outbuf, outbufsize - outbytesleft + 1);
     free(outbuf);
     return convstr;
 }
