@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -33,10 +33,8 @@
 // XIM Server supporting CJK languages
 // initialize many modules
 
-#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
-
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <stdio.h>
@@ -151,7 +149,7 @@ static void main_loop()
 
 	std::map<int, fd_watch_struct>::iterator it;
 	int  fd_max = 0;
-	for (it = fd_watch_stat.begin(); it != fd_watch_stat.end(); it++) {
+	for (it = fd_watch_stat.begin(); it != fd_watch_stat.end(); ++it) {
 	    int fd = it->first;
 	    if (it->second.mask & READ_OK)
 		FD_SET(fd, &rfds);
@@ -165,12 +163,21 @@ static void main_loop()
 	    continue;
 	}
 
-	for (it = fd_watch_stat.begin(); it != fd_watch_stat.end(); it++) {
+	it = fd_watch_stat.begin();
+	while (it != fd_watch_stat.end()) {
 	    int fd = it->first;
 	    if (FD_ISSET(fd, &rfds))
 		it->second.fn(fd, READ_OK);
 	    if (FD_ISSET(fd, &wfds))
 		it->second.fn(fd, WRITE_OK);
+	    // fd_watch_stat may be modified by above functions at
+	    // this point.  Since the behavior with incrementing
+	    // invalidated iterator is compiler dependent, use safer
+	    // way.
+	    it = fd_watch_stat.find(fd);
+	    if (it == fd_watch_stat.end())	// shouldn't happen
+		break;
+	    ++it;
 	}
     }
 }
@@ -326,9 +333,8 @@ check_pending_xevent(void)
 }
 
 static void
-xEventRead(int fd, int ev)
+xEventRead(int /* fd */, int /* ev */)
 {
-    XFlush(XimServer::gDpy);
     check_pending_xevent();
 }
 
@@ -396,7 +402,7 @@ print_uim_info()
     if (uim_info.empty())
 	printf("  None.\n");
     else
-	for (it = uim_info.begin(); it != uim_info.end(); it++)
+	for (it = uim_info.begin(); it != uim_info.end(); ++it)
 	    printf("  %s (%s)\n", it->name, it->lang);
     
 }
@@ -405,7 +411,7 @@ static void
 clear_uim_info()
 {
     std::list<UIMInfo>::iterator it;
-    for (it = uim_info.begin(); it != uim_info.end(); it++) {
+    for (it = uim_info.begin(); it != uim_info.end(); ++it) {
 	free(it->name);
 	free(it->lang);
 	free(it->desc);
@@ -485,7 +491,7 @@ static void check_default_engine(const char *locale)
     bool found = false;
     if (default_engine) {
 	std::list<UIMInfo>::iterator it;
-	for (it = uim_info.begin(); it != uim_info.end(); it++) {
+	for (it = uim_info.begin(); it != uim_info.end(); ++it) {
 	    if (!strcmp(it->name, default_engine)) {
 		found = true;
 		break;
@@ -517,7 +523,7 @@ terminate_x_connection()
 }
 
 void
-reload_uim(int x)
+reload_uim(int /* x */)
 {
     fprintf(stderr, "\nReloading uim...\n\n");
 
@@ -528,9 +534,9 @@ reload_uim(int x)
     std::map<Window, XimServer *>::iterator it;
     std::list<InputContext *>::iterator it_c;
 
-    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); it++) {
+    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it) {
 	XimServer *xs = it->second;
-	for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); it_c++)
+	for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); ++it_c)
 	    (*it_c)->clear();
     }
 
@@ -539,9 +545,9 @@ reload_uim(int x)
     get_uim_info();
     //print_uim_info();
 
-    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); it++) {
+    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it) {
 	XimServer *xs = it->second;
-	for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); it_c++) {
+	for (it_c = xs->ic_list.begin(); it_c != xs->ic_list.end(); ++it_c) {
 	    const char *engine = (*it_c)->get_engine_name();
 	    (*it_c)->createUimContext(engine);
 	}
@@ -602,7 +608,7 @@ main(int argc, char **argv)
 
     // First, setup conversion engine selected by cmdline option or
     // "default-im-name" on ~/.uim.
-    for (it = uim_info.begin(); it != uim_info.end(); it++) {
+    for (it = uim_info.begin(); it != uim_info.end(); ++it) {
 	if (strcmp(it->name, default_engine) == 0) {
 	    XimServer *xs = new XimServer(it->name, it->lang);
 	    res = xs->setupConnection(true);
