@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -33,7 +33,7 @@
 // miscellaneous functions
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <stdio.h>
@@ -472,7 +472,7 @@ static void fmtstr(char *buffer, size_t *currlen, size_t maxlen,
     printf("fmtstr min=%d max=%d s=[%s]\n", min, max, value);
 #endif
     if (value == 0) {
-	value = "<NULL>";
+	value = (char *)"<NULL>";
     }
 
     for (strln = 0; value[strln]; ++strln); /* strlen */
@@ -626,7 +626,7 @@ static double my_modf(double x0, double *iptr)
     double x = x0;
     double f = 1.0;
 
-    for (i=0;i<100;i++) {
+    for (i = 0; i < 100; i++) {
 	l = (long)x;
 	if (l <= (x+1) && l >= (x-1)) break;
 	x *= 0.1;
