@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -30,10 +30,8 @@
   SUCH DAMAGE.
 */
 
-#define _GNU_SOURCE // for asprintf on stdio.h with old glibc/gcc
-
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <stdio.h>
@@ -54,6 +52,7 @@
 #include "uim/uim-helper.h"
 #include "uim/uim-im-switcher.h"
 #include "uim/uim-compat-scm.h"
+#include "uim/uim-stdint.h"
 
 #ifndef __GNUC__
 # ifdef HAVE_ALLOCA_H
@@ -66,8 +65,7 @@ extern Atom xim_servers;
 InputContext *InputContext::mFocusedContext = NULL;
 
 static int check_modifier(std::list<KeySym> list);
-static int gShiftMask, gLockMask, gControlMask, gMod1Mask,
-	   gMod2Mask, gMod3Mask, gMod4Mask, gMod5Mask;
+static int gMod1Mask, gMod2Mask, gMod3Mask, gMod4Mask, gMod5Mask;
 static int gXNumLockMask;
 
 
@@ -78,7 +76,7 @@ void print_ustring(uString *s)
     uchar ch;
     char utf8[6];
     int nbyte;
-    for (i = s->begin(); i != s->end(); i++) {
+    for (i = s->begin(); i != s->end(); ++i) {
 	ch = *i;
 	nbyte = utf8_wctomb((unsigned char *)utf8, ch);
 	utf8[nbyte] = '\0';
@@ -95,7 +93,7 @@ void erase_ustring(uString *s)
 void append_ustring(uString *d, uString *s)
 {
     uString::iterator i;
-    for (i = s->begin(); i !=s->end(); i++) {
+    for (i = s->begin(); i !=s->end(); ++i) {
 	d->push_back(*i);
     }
 }
@@ -116,7 +114,7 @@ InputContext *XimServer::createContext(XimIC *xic, const char *engine)
 void XimServer::deleteContext(InputContext *ic)
 {
     std::list<InputContext *>::iterator it;
-    for (it = ic_list.begin(); it != ic_list.end(); it++) {
+    for (it = ic_list.begin(); it != ic_list.end(); ++it) {
 	if (*it == ic) {
 	    ic_list.erase(it);
 	    break;
@@ -127,7 +125,7 @@ void XimServer::deleteContext(InputContext *ic)
 void XimServer::changeContext(const char *engine) {
     set_im(engine);
     std::list<InputContext *>::iterator it;
-    for (it = ic_list.begin(); it != ic_list.end(); it++) {
+    for (it = ic_list.begin(); it != ic_list.end(); ++it) {
 	(*it)->changeContext(engine);
     }
     // make sure to use appropriate locale for the focused context
@@ -138,7 +136,7 @@ void XimServer::changeContext(const char *engine) {
 
 void XimServer::customContext(const char *custom, const char *val) {
     std::list<InputContext *>::iterator it;
-    for (it = ic_list.begin(); it != ic_list.end(); it++) {
+    for (it = ic_list.begin(); it != ic_list.end(); ++it) {
 	(*it)->customContext(custom, val);
 	break;
     }
@@ -225,7 +223,7 @@ XimServer::setupConnection(bool useDefaultIM)
     XGetWindowProperty(XimServer::gDpy, DefaultRootWindow(XimServer::gDpy),
 		       xim_servers, 0, 8192 ,False,
 		       XA_ATOM, &type, &format,
-		       &nr_prop, &nr_bytes, (unsigned char **)&prop);
+		       &nr_prop, &nr_bytes, (unsigned char **)(uintptr_t)&prop);
     int i;
     if (type != XA_ATOM || format != 32)
 	mode = PropModeReplace;
@@ -299,7 +297,7 @@ void XimServer::set_im(const char *engine)
 const char *get_im_lang_from_engine(const char *engine)
 {
     std::list<UIMInfo>::iterator it;
-    for (it = uim_info.begin(); it != uim_info.end(); it++) {
+    for (it = uim_info.begin(); it != uim_info.end(); ++it) {
 	if (!strcmp(it->name, engine))
 	    return it->lang;
     }
@@ -406,10 +404,15 @@ InputContext::createUimContext(const char *engine)
 			InputContext::candidate_deactivate_cb);
 	uim_set_prop_list_update_cb(uc,
 			InputContext::update_prop_list_cb);
+#if 0
 	uim_set_prop_label_update_cb(uc,
 			InputContext::update_prop_label_cb);
+#endif
 	uim_set_configuration_changed_cb(uc,
 			InputContext::configuration_changed_cb);
+	uim_set_im_switch_request_cb(uc,
+			InputContext::switch_app_global_im_cb,
+			InputContext::switch_system_global_im_cb);
 
 	if (mFocusedContext == this)
 	    uim_prop_list_update(uc);
@@ -420,8 +423,13 @@ InputContext::createUimContext(const char *engine)
 void
 InputContext::changeContext(const char *engine)
 {
-    const char *encoding = mXic->get_encoding();
-    const char *im_lang = get_im_lang_from_engine(engine);
+    const char *encoding, *im_lang;
+
+    if (!strcmp(mEngineName, engine))
+	return;
+
+    encoding = mXic->get_encoding();
+    im_lang = get_im_lang_from_engine(engine);
 
     // Don't change im unless encoding matches for clients with legacy locales.
     if (strcmp(encoding, "UTF-8")) {
@@ -446,14 +454,38 @@ void InputContext::configuration_changed()
     const char *engine = uim_get_current_im_name(mUc);
 
     review_im(engine);
+
+    InputContext *focusedContext = InputContext::focusedContext();
+    if (this == focusedContext)
+	send_im_list();
+}
+
+void InputContext::switch_app_global_im(const char *name)
+{
+    get_im_by_id(this->get_ic()->get_imid())->changeContext(name);
+}
+
+void InputContext::switch_system_global_im(const char *name)
+{
+    char *msg;
+    std::map<Window, XimServer *>::iterator it;
+
+    for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it)
+	(*it).second->changeContext(name);
+
+    asprintf(&msg, "im_change_whole_desktop\n%s\n", name);
+    uim_helper_send_message(lib_uim_fd, msg);
+    free(msg);
 }
 
 void InputContext::review_im(const char *engine)
 {
-    char *locale;
+    char *locale, *prev_engine;
     const char *client_locale, *engine_locales;
     const char *encoding;
 
+    prev_engine = mEngineName;
+    mEngineName = strdup(engine);
     encoding = mXic->get_encoding();
     client_locale = mXic->get_lang_region();
     engine_locales = compose_localenames_from_im_lang(get_im_lang_from_engine(engine));
@@ -468,16 +500,11 @@ void InputContext::review_im(const char *engine)
 	setlocale(LC_CTYPE, locale);
 	free(mLocaleName);
 	mLocaleName = locale;
-	free(mEngineName);
-	mEngineName = strdup(engine);
     } else {
 	if (!is_locale_included(engine_locales, client_locale))
-	    changeContext(mEngineName);
-	else {
-	    free(mEngineName);
-	    mEngineName = strdup(engine);
-	}
+	    changeContext(prev_engine);
     }
+    free(prev_engine);
 }
 
 void
@@ -618,6 +645,20 @@ void InputContext::configuration_changed_cb(void *ptr)
     ic->configuration_changed();
 }
 
+void InputContext::switch_app_global_im_cb(void *ptr, const char *name)
+{
+    InputContext *ic = (InputContext *)ptr;
+
+    ic->switch_app_global_im(name);
+}
+
+void InputContext::switch_system_global_im_cb(void *ptr, const char *name)
+{
+    InputContext *ic = (InputContext *)ptr;
+
+    ic->switch_system_global_im(name);
+}
+
 void InputContext::clear_pe_stat()
 {
     m_pe->clear();
@@ -661,7 +702,7 @@ void InputContext::pushback_preedit_string(int attr, const char *str)
     mServer->strToUstring(&js, str);
 
     uString::iterator it;
-    for (it = js.begin(); it != js.end(); it++) {
+    for (it = js.begin(); it != js.end(); ++it) {
 	m_pe->push_uchar(*it);
     }
 }
@@ -827,7 +868,7 @@ void InputContext::candidate_deactivate()
 	Canddisp *disp = canddisp_singleton();
 
 	disp->deactivate();
-	for (i = active_candidates.begin(); i != active_candidates.end(); i++) {
+	for (i = active_candidates.begin(); i != active_candidates.end(); ++i) {
 	    free((char *)*i);
 	}
 	mCandwinActive = false;
@@ -835,6 +876,42 @@ void InputContext::candidate_deactivate()
     }
 }
 
+char *InputContext::get_caret_state_label_from_prop_list(const char *str)
+{
+    const char *p, *q;
+    char *state_label = NULL;
+    char label[10];
+    int len, state_label_len = 0;
+
+    p = str;
+    while ((p = strstr(p, "branch\t"))) {
+	p = strchr(p + 7, '\t');
+	if (p) {
+	    p++;
+	    q = strchr(p, '\t');
+	    len = q - p;
+	    if (q && len < 10) {
+		strlcpy(label, p, len + 1);
+		if (!state_label) {
+		    state_label_len = len;
+		    state_label = strdup(label);
+		} else {
+		    state_label_len += (len + 1);
+		    state_label = (char *)realloc(state_label,
+						      state_label_len + 1);
+		    if (state_label) {
+			strcat(state_label, "\t");
+			strcat(state_label, label);
+			state_label[state_label_len] = '\0';
+		    }
+		}
+	    }
+	}
+    }
+
+    return state_label;
+}
+
 void InputContext::update_prop_list(const char *str)
 {
     char *buf;
@@ -844,25 +921,46 @@ void InputContext::update_prop_list(const char *str)
 	return;
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
+
+#if 1
+    // Show caret state indicator with this function instead of
+    // InputContext::update_prop_label() to workaround the label
+    // mismatch during IM switch caused from context-update-widgets.
+    uim_bool show_caret_state =
+	uim_scm_symbol_value_bool("bridge-show-input-state?");
+    if (show_caret_state == UIM_TRUE) {
+	char *label;
+	int timeout;
+	Canddisp *disp = canddisp_singleton();
+
+	timeout =
+	    uim_scm_symbol_value_int("bridge-show-input-state-time-length");
+	label = get_caret_state_label_from_prop_list(str);
+	disp->show_caret_state(label, timeout);
+	free(label);
+	mCaretStateShown = true;
+    }
+#endif
 }
 
 void InputContext::update_prop_label(const char *str)
 {
     char *buf;
-    uim_bool show_caret_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
 
     asprintf(&buf, "prop_label_update\ncharset=UTF-8\n%s", str);
     if (!buf)
 	return;
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
-    
+#if 0    
+    uim_bool show_caret_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
     if (show_caret_state == UIM_TRUE) {
 	int timeout = uim_scm_symbol_value_int("bridge-show-input-state-time-length");
 	Canddisp *disp = canddisp_singleton();
 	disp->show_caret_state(str, timeout);
 	mCaretStateShown = true;
     }
+#endif
 }
 
 const char *InputContext::get_engine_name()
@@ -885,10 +983,7 @@ keyState::keyState(XimIC *ic)
     XimIM *im;
     DefTree *top;
 
-    mAltOn = false;
-    mMetaOn = false;
-    mSuperOn = false;
-    mHyperOn = false;
+    mModState = 0;
     mIc = ic;
 
     im = get_im_by_id(mIc->get_imid());
@@ -908,32 +1003,31 @@ void keyState::check_key(keyEventX *x)
     mXKeySym = x->key_sym;
     mXKeyState = x->state;
 
+    mPreModState = mModState;
+
     if (x->press) {
 	m_bPush = true;
 
-	if (!(g_option_mask & OPT_ON_DEMAND_SYNC)) {
-	    // Only KeyPress is forwarded with full-synchronous
-	    // method.  So reset modifiers here.
-	    if (!(x->state) || x->state == LockMask || x->state == gXNumLockMask)
-		mAltOn = mMetaOn = mSuperOn = mHyperOn = false;
-	}
+	if (!(x->state) || x->state == LockMask || x->state == gXNumLockMask)
+	    mModState = mPreModState = 0;
 
+	mPreModState = mModState;
 	switch (x->key_sym) {
 	case XK_Alt_L:
 	case XK_Alt_R:
-	    mAltOn = true;
+	    mModState |= UMod_Alt;
 	    break;
 	case XK_Meta_L:
 	case XK_Meta_R:
-	    mMetaOn = true;
+	    mModState |= UMod_Meta;
 	    break;
 	case XK_Super_L:
 	case XK_Super_R:
-	    mSuperOn = true;
+	    mModState |= UMod_Super;
 	    break;
 	case XK_Hyper_L:
 	case XK_Hyper_R:
-	    mHyperOn = true;
+	    mModState |= UMod_Hyper;
 	    break;
 	default:
 	    break;
@@ -944,19 +1038,19 @@ void keyState::check_key(keyEventX *x)
 	switch (x->key_sym) {
 	case XK_Alt_L:
 	case XK_Alt_R:
-	    mAltOn = false;
+	    mModState &= ~UMod_Alt;
 	    break;
 	case XK_Meta_L:
 	case XK_Meta_R:
-	    mMetaOn = false;
+	    mModState &= ~UMod_Meta;
 	    break;
 	case XK_Super_L:
 	case XK_Super_R:
-	    mSuperOn = false;
+	    mModState &= ~UMod_Super;
 	    break;
 	case XK_Hyper_L:
 	case XK_Hyper_R:
-	    mHyperOn = false;
+	    mModState &= ~UMod_Hyper;
 	    break;
 	default:
 	    break;
@@ -964,21 +1058,19 @@ void keyState::check_key(keyEventX *x)
     }
 
     if (x->state & ShiftMask)
-	mModifier |= gShiftMask;
-    if (x->state & LockMask)
-	mModifier |= gLockMask;
+	mModifier |= UMod_Shift;
     if (x->state & ControlMask)
-	mModifier |= gControlMask;
+	mModifier |= UMod_Control;
     if (x->state & Mod1Mask)
-	mModifier |= revise_mod(gMod1Mask);
+	mModifier |= (gMod1Mask & mPreModState);
     if (x->state & Mod2Mask)
-	mModifier |= revise_mod(gMod2Mask);
+	mModifier |= (gMod2Mask & mPreModState);
     if (x->state & Mod3Mask)
-	mModifier |= revise_mod(gMod3Mask);
+	mModifier |= (gMod3Mask & mPreModState);
     if (x->state & Mod4Mask)
-	mModifier |= revise_mod(gMod4Mask);
+	mModifier |= (gMod4Mask & mPreModState);
     if (x->state & Mod5Mask)
-	mModifier |= revise_mod(gMod5Mask);
+	mModifier |= (gMod5Mask & mPreModState);
 
     if (x->key_sym < 128 && x->key_sym >= 32)
 	mKey = x->key_sym;
@@ -1001,14 +1093,24 @@ void keyState::check_key(keyEventX *x)
 	case XK_Next: mKey = UKey_Next; break;
 	case XK_Home: mKey = UKey_Home; break;
 	case XK_End: mKey = UKey_End; break;
-	case XK_Kanji:
-	case XK_Zenkaku_Hankaku: mKey = UKey_Zenkaku_Hankaku; break;
-	// Don't forward Multi_key to libuim.
-	//case XK_Multi_key: mKey = UKey_Multi_key; break;
-	case XK_Multi_key: mKey = UKey_Other; break;
+	case XK_Multi_key: mKey = UKey_Multi_key; break;
 	case XK_Mode_switch: mKey = UKey_Mode_switch; break;
-	case XK_Henkan_Mode: mKey = UKey_Henkan_Mode; break;
+	case XK_Kanji: mKey = UKey_Kanji; break;
 	case XK_Muhenkan: mKey = UKey_Muhenkan; break;
+	case XK_Henkan_Mode: mKey = UKey_Henkan_Mode; break;
+	case XK_Romaji: mKey = UKey_Romaji; break;
+	case XK_Hiragana: mKey = UKey_Hiragana; break;
+	case XK_Katakana: mKey = UKey_Katakana; break;
+	case XK_Hiragana_Katakana: mKey = UKey_Hiragana_Katakana; break;
+	case XK_Zenkaku: mKey = UKey_Zenkaku; break;
+	case XK_Hankaku: mKey = UKey_Hankaku; break;
+	case XK_Zenkaku_Hankaku: mKey = UKey_Zenkaku_Hankaku; break;
+	case XK_Touroku: mKey = UKey_Touroku; break;
+	case XK_Massyo: mKey = UKey_Massyo; break;
+	case XK_Kana_Lock: mKey = UKey_Kana_Lock; break;
+	case XK_Kana_Shift: mKey = UKey_Kana_Shift; break;
+	case XK_Eisu_Shift: mKey = UKey_Eisu_Shift; break;
+	case XK_Eisu_toggle: mKey = UKey_Eisu_toggle; break;
 	case XK_Shift_L: mKey = UKey_Shift_key; break;
 	case XK_Shift_R: mKey = UKey_Shift_key; break;
 	case XK_Control_L: mKey = UKey_Control_key; break;
@@ -1047,20 +1149,6 @@ bool keyState::is_push()
     return m_bPush;
 }
 
-int keyState::revise_mod(int uim_mod)
-{
-    if ((uim_mod & UMod_Alt) && (mAltOn == false))
-	uim_mod &= ~UMod_Alt;
-    if ((uim_mod & UMod_Meta) && (mMetaOn == false))
-	uim_mod &= ~UMod_Meta;
-    if ((uim_mod & UMod_Super) && (mSuperOn == false))
-	uim_mod &= ~UMod_Super;
-    if ((uim_mod & UMod_Hyper) && (mHyperOn == false))
-	uim_mod &= ~UMod_Hyper;
-
-    return uim_mod;
-}
-
 KeySym keyState::xkeysym()
 {
     return mXKeySym;
@@ -1073,7 +1161,7 @@ int keyState::xkeystate()
 
 void keyState::reset()
 {
-    mAltOn = mMetaOn = mHyperOn = mSuperOn = false;
+    mModState = 0;
     mCompose->reset();
 }
 
@@ -1087,16 +1175,8 @@ static int check_modifier(std::list<KeySym> keysym_list)
 {
     int ret = 0;
     std::list<KeySym>::iterator i;
-    for (i = keysym_list.begin(); i != keysym_list.end(); i++) {
+    for (i = keysym_list.begin(); i != keysym_list.end(); ++i) {
 	switch (*i) {
-	case XK_Shift_L:
-	case XK_Shift_R:
-	    ret |= UMod_Shift;
-	    break;
-	case XK_Control_L:
-	case XK_Control_R:
-	    ret |= UMod_Control;
-	    break;
 	case XK_Alt_L:
 	case XK_Alt_R:
 	    ret |= UMod_Alt;
@@ -1124,8 +1204,7 @@ void init_modifier_keys() {
     int i, k = 0;
     int min_keycode, max_keycode, keysyms_per_keycode = 0;
 
-    std::list<KeySym> ShiftMaskSyms, LockMaskSyms, ControlMaskSyms,
-		      Mod1MaskSyms, Mod2MaskSyms, Mod3MaskSyms,
+    std::list<KeySym> Mod1MaskSyms, Mod2MaskSyms, Mod3MaskSyms,
 		      Mod4MaskSyms, Mod5MaskSyms;
 
     gXNumLockMask = 0;
@@ -1146,9 +1225,9 @@ void init_modifier_keys() {
 		} while (!ks && index < keysyms_per_keycode);
 
 		switch (i) {
-		case ShiftMapIndex: ShiftMaskSyms.push_back(ks); break;
-		case LockMapIndex: LockMaskSyms.push_back(ks); break;
-		case ControlMapIndex: ControlMaskSyms.push_back(ks); break;
+		case ShiftMapIndex: break;
+		case LockMapIndex: break;
+		case ControlMapIndex: break;
 		case Mod1MapIndex: Mod1MaskSyms.push_back(ks); break;
 		case Mod2MapIndex: Mod2MaskSyms.push_back(ks); break;
 		case Mod3MapIndex: Mod3MaskSyms.push_back(ks); break;
@@ -1166,9 +1245,6 @@ void init_modifier_keys() {
     XFreeModifiermap(map);
     XFree(sym);
 
-    gShiftMask = check_modifier(ShiftMaskSyms);
-    gLockMask = check_modifier(LockMaskSyms);
-    gControlMask = check_modifier(ControlMaskSyms);
     gMod1Mask = check_modifier(Mod1MaskSyms);
     gMod2Mask = check_modifier(Mod2MaskSyms);
     gMod3Mask = check_modifier(Mod3MaskSyms);
