@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -133,16 +133,13 @@ public:
     bool is_push(); // for distinguish from release
     void print();
 private:
-    int revise_mod(int uim_mod);
     int mKey;
     int mModifier;
     KeySym mXKeySym;
     int mXKeyState;
     bool m_bPush;
-    bool mAltOn;
-    bool mMetaOn;
-    bool mHyperOn;
-    bool mSuperOn;
+    int mModState;
+    int mPreModState;
     Compose *mCompose;
     XimIC *mIc;
 };
@@ -182,6 +179,8 @@ public:
     void customContext(const char *custom, const char *val);
     void createUimContext(const char *engine);
     void configuration_changed();
+    void switch_app_global_im(const char *name);
+    void switch_system_global_im(const char *name);
 public:
     static void commit_cb(void *ptr, const char *str);
     static void clear_cb(void *ptr);
@@ -194,12 +193,15 @@ public:
     static void update_prop_list_cb(void *ptr, const char *str);
     static void update_prop_label_cb(void *ptr, const char *str);
     static void configuration_changed_cb(void *ptr);
+    static void switch_app_global_im_cb(void *ptr, const char *name);
+    static void switch_system_global_im_cb(void *ptr, const char *name);
     static InputContext *focusedContext();
     static void deletefocusedContext();
 private:
     void commit_string(char *);
     void clear_pe_stat();
     void review_im(const char *engine);
+    char *get_caret_state_label_from_prop_list(const char *str);
 
     XimIC *mXic;
     XimServer *mServer;
